(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  if (ELA.Models == null) {
    ELA.Models = {};
  }

  ELA.Models.Curve = (function(superClass) {
    var Presenter;

    extend(Curve, superClass);

    function Curve() {
      this.toJSON = bind(this.toJSON, this);
      this.deselect = bind(this.deselect, this);
      this.select = bind(this.select, this);
      return Curve.__super__.constructor.apply(this, arguments);
    }

    Curve.prototype.defaults = {
      selected: false,
      lineDash: null,
      lineWidth: 3,
      zIndex: 0,
      group: 'general'
    };

    Curve.prototype.parentCurve = null;

    Curve.prototype.LINE_DASHES = [[], [6, 4], [2, 2], [2, 4, 10, 4], [2, 4, 2, 4, 2, 4, 2, 4, 10, 4], [26, 6], [2, 4, 26, 4], [2, 4, 2, 4, 2, 4, 2, 4, 26, 4]];

    Curve.prototype.initialize = function(attributes, options) {
      var name, ref, subcurve, subcurves;
      if (options == null) {
        options = {};
      }
      Curve.__super__.initialize.apply(this, arguments);
      subcurves = {};
      ref = attributes.subcurves || {};
      for (name in ref) {
        subcurve = ref[name];
        subcurves[name] = new ELA.Models.Curve(subcurve, {
          parentCurve: this
        });
      }
      this.set('subcurves', subcurves);
      if (options.parentCurve != null) {
        return this.parentCurve = options.parentCurve;
      }
    };

    Curve.prototype.showInLegend = function() {
      var show;
      show = this.get('showInLegend');
      if (show != null) {
        return show;
      } else {
        return true;
      }
    };

    Curve.prototype.strokeStyle = function() {
      var distance, hue, index, length, luminance, saturation;
      if (this.get('strokeStyle') != null) {
        return this.get('strokeStyle');
      } else if (this.parentCurve != null) {
        return this.parentCurve.strokeStyle();
      } else {
        index = this.collection.filter(function(c) {
          return c.attributes.strokeStyle === void 0;
        }).indexOf(this);
        length = this.collection.length;
        saturation = Math.round(65 - 40 * ((index % 20) / 20));
        luminance = Math.round(65 - 15 * ((index % 20) / 20 + 1));
        distance = 360 * 0.61088;
        hue = index * distance;
        return 'hsl(' + hue + ("," + saturation + "%," + luminance + "%)");
      }
    };

    Curve.prototype.lineDash = function(idx) {
      if (idx == null) {
        idx = 0;
      }
      return this.get('lineDash') || this.LINE_DASHES[idx % this.LINE_DASHES.length];
    };

    Curve.prototype.hasSubcurves = function() {
      return _.keys(this.subcurves()).length > 0;
    };

    Curve.prototype.subcurves = function() {
      return this.get('subcurves');
    };

    Curve.prototype.subcurve = function(key) {
      return this.get('subcurves')[key];
    };

    Curve.prototype.select = function() {
      return this.set({
        selected: true
      });
    };

    Curve.prototype.deselect = function() {
      return this.set({
        selected: false
      });
    };

    Curve.prototype.toJSON = function() {
      return _.defaults(Curve.__super__.toJSON.call(this), {
        strokeStyle: this.strokeStyle(),
        hsl: this.strokeStyle()
      });
    };

    Curve.prototype.Presenter = Presenter = (function(superClass1) {
      extend(Presenter, superClass1);

      function Presenter() {
        return Presenter.__super__.constructor.apply(this, arguments);
      }

      Presenter.prototype.unitValue = function(value) {
        var unitFactor, unitFn;
        unitFn = this.model.get('unitFn');
        if (_.isFunction(unitFn)) {
          return unitFn(value);
        }
        unitFactor = this.model.get('unitFactor');
        if (unitFactor != null) {
          return unitFactor * value;
        }
        return value;
      };

      Presenter.prototype.label = function() {
        return this._label || (this._label = (function(_this) {
          return function() {
            var func;
            func = _this.model.get('function');
            return _this.loadLocale("curves." + func + ".label", "curves." + func + ".yAxisLabel", "curves." + func + ".xAxisLabel", 'curves.yAxisLabel', 'curves.xAxisLabel', {
              defaultValue: func.toLabel()
            });
          };
        })(this)());
      };

      Presenter.prototype.xAxisLabel = function() {
        return this._xAxisLabel || (this._xAxisLabel = (function(_this) {
          return function() {
            var func;
            func = _this.model.get('function');
            return _this.loadLocale("curves." + func + ".xAxisLabel", 'curves.xAxisLabel', {
              defaultValue: func.toLabel()
            });
          };
        })(this)());
      };

      Presenter.prototype.yAxisLabel = function() {
        return this._yAxisLabel || (this._yAxisLabel = (function(_this) {
          return function() {
            var func;
            func = _this.model.get('function');
            return _this.loadLocale("curves." + func + ".yAxisLabel", 'curves.yAxisLabel', {
              defaultValue: func.toLabel()
            });
          };
        })(this)());
      };

      Presenter.prototype.unitLabel = function() {
        var unit;
        unit = this.model.get('unit');
        return this._unitLabel != null ? this._unitLabel : this._unitLabel = (function(_this) {
          return function() {
            var func, l;
            func = _this.model.get('function');
            l = _this.loadLocale("curves." + func + ".unitLabel", 'curves.unitLabel', {
              returnNull: true
            });
            return l || _this.yAxisUnitLabel() || _this.xAxisUnitLabel();
          };
        })(this)();
      };

      Presenter.prototype.xAxisUnitLabel = function() {
        return this._xAxisUnitLabel || (this._xAxisUnitLabel = (function(_this) {
          return function() {
            var func;
            func = _this.model.get('function');
            return _this.loadLocale("curves." + func + ".xAxisUnitLabel", 'curves.xAxisUnitLabel', {
              returnNull: true
            });
          };
        })(this)());
      };

      Presenter.prototype.yAxisUnitLabel = function() {
        return this._yAxisUnitLabel || (this._yAxisUnitLabel = (function(_this) {
          return function() {
            var func;
            func = _this.model.get('function');
            return _this.loadLocale("curves." + func + ".yAxisUnitLabel", 'curves.yAxisUnitLabel', {
              returnNull: true
            });
          };
        })(this)());
      };

      Presenter.prototype.fullLabel = function() {
        var unit;
        unit = this.unitLabel();
        unit = unit != null ? " [" + unit + "]" : '';
        return "" + (this.label()) + unit;
      };

      Presenter.prototype.fullXAxisLabel = function() {
        var unit;
        unit = this.xAxisUnitLabel();
        unit = unit != null ? " [" + unit + "]" : '';
        return "" + (this.xAxisLabel()) + unit;
      };

      Presenter.prototype.fullYAxisLabel = function() {
        var unit;
        unit = this.yAxisUnitLabel();
        unit = unit != null ? " [" + unit + "]" : '';
        return "" + (this.yAxisLabel()) + unit;
      };

      Presenter.prototype.subcurveLabel = function(subcurve) {
        var func;
        func = this.model.get('function');
        return this.loadLocale("curves." + func + ".subcurves." + subcurve + ".label", {
          defaultValue: func
        });
      };

      return Presenter;

    })(Backbone.Poised.View);

    return Curve;

  })(Backbone.Model);

}).call(this);
