(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  if (ELA.Views == null) {
    ELA.Views = {};
  }

  ELA.Views.AxisHandler = (function(superClass) {
    extend(AxisHandler, superClass);

    function AxisHandler() {
      this.render = bind(this.render, this);
      this.renderSpan = bind(this.renderSpan, this);
      this.text = bind(this.text, this);
      this.handleFocusOut = bind(this.handleFocusOut, this);
      this.closeInputField = bind(this.closeInputField, this);
      this.focusInputField = bind(this.focusInputField, this);
      this.updateValue = bind(this.updateValue, this);
      this.handleInputTap = bind(this.handleInputTap, this);
      this.handlePan = bind(this.handlePan, this);
      this.handleTap = bind(this.handleTap, this);
      return AxisHandler.__super__.constructor.apply(this, arguments);
    }

    AxisHandler.prototype.className = 'axis-handler';

    AxisHandler.prototype.events = {
      'pan': 'handlePan',
      'tap': 'handleTap',
      'press': 'showInputField',
      'doubletap': 'showInputField',
      'pressup': 'focusInputField',
      'tap input': 'handleInputTap'
    };

    AxisHandler.prototype.hammerjs = {
      recognizers: [
        [
          Hammer.Pan, {
            direction: Hammer.DIRECTION_ALL,
            threshold: 1
          }
        ], [
          Hammer.Tap, {
            threshold: 1
          }
        ], [
          Hammer.Tap, {
            event: 'doubletap',
            taps: 2,
            posThreshold: 20,
            threshold: 1
          }, ['tap']
        ], [
          Hammer.Press, {
            threshold: 1
          }
        ]
      ]
    };

    AxisHandler.prototype.initialize = function(options) {
      var ref, ref1, ref10, ref11, ref12, ref13, ref14, ref15, ref16, ref17, ref18, ref2, ref3, ref4, ref5, ref6, ref7, ref8, ref9;
      this.displayParams = options.displayParams;
      this.position = options.position || 'bottom';
      this.orientation = (function() {
        switch (this.position) {
          case 'bottom':
            return 'x';
          case 'left':
            return 'y';
        }
      }).call(this);
      this.attribute = options.attribute;
      this.precision = (ref = ELA.settings[this.model.name]) != null ? (ref1 = ref.formFields) != null ? (ref2 = ref1[this.attribute]) != null ? ref2.precision : void 0 : void 0 : void 0;
      if (this.precision == null) {
        this.precision = options.precision;
      }
      if (this.precision == null) {
        this.precision = 2;
      }
      this.maxValue = (ref3 = ELA.settings[this.model.name]) != null ? (ref4 = ref3.formFields) != null ? (ref5 = ref4[this.attribute]) != null ? ref5.maxValue : void 0 : void 0 : void 0;
      if (this.maxValue == null) {
        this.maxValue = (ref6 = ELA.settings[this.model.name]) != null ? (ref7 = ref6.formFields) != null ? (ref8 = ref7[this.attribute]) != null ? (ref9 = ref8.range) != null ? ref9[1] : void 0 : void 0 : void 0 : void 0;
      }
      if (this.maxValue == null) {
        this.maxValue = options.maxValue;
      }
      if (this.maxValue == null) {
        this.maxValue = (ref10 = options.range) != null ? ref10[1] : void 0;
      }
      this.minValue = (ref11 = ELA.settings[this.model.name]) != null ? (ref12 = ref11.formFields) != null ? (ref13 = ref12[this.attribute]) != null ? ref13.minValue : void 0 : void 0 : void 0;
      if (this.minValue == null) {
        this.minValue = (ref14 = ELA.settings[this.model.name]) != null ? (ref15 = ref14.formFields) != null ? (ref16 = ref15[this.attribute]) != null ? (ref17 = ref16.range) != null ? ref17[0] : void 0 : void 0 : void 0 : void 0;
      }
      if (this.minValue == null) {
        this.minValue = options.minValue;
      }
      if (this.minValue == null) {
        this.minValue = (ref18 = options.range) != null ? ref18[1] : void 0;
      }
      this.listenTo(this.model, "change:" + this.attribute, this.renderSpan);
      this.subviews = {};
      this.formSettings = this.model.loadSettings("formFields")[this.attribute];
      this.params = new Backbone.Model({
        showInput: false
      });
      this.listenTo(this.params, 'change:showInput', this.renderSpan);
      return this.lastFocusOutAt = void 0;
    };

    AxisHandler.prototype.handleTap = function(e) {
      if (!this.hasRecentlyOpenInput()) {
        return this.updateValue(e);
      }
    };

    AxisHandler.prototype.handlePan = function(e) {
      if (!this.params.get('showInput')) {
        return this.updateValue(e);
      }
    };

    AxisHandler.prototype.handleInputTap = function(e) {
      return e.stopPropagation();
    };

    AxisHandler.prototype.updateValue = function(e) {
      var graphOffset, height, origin, point, pow, range, width, y;
      origin = this.displayParams.get(this.orientation + "Origin");
      range = this.displayParams.get(this.orientation + "Range");
      graphOffset = this.parentView.subviews.graph.$el.offset();
      if (this.orientation === 'x') {
        width = this.displayParams.get('width');
        point = (range / width) * (e.gesture.center.x - graphOffset.left - origin);
      } else {
        height = this.displayParams.get('height');
        y = e.gesture.center.y - graphOffset.top;
        y = Math.min(Math.max(y, 0), height);
        point = (range / height) * (origin - y);
      }
      pow = Math.pow(10, this.precision);
      point = Math.round(point * pow) / pow;
      if (isFinite(this.minValue)) {
        point = Math.max(point, this.minValue);
      }
      if (isFinite(this.maxValue)) {
        point = Math.min(point, this.maxValue);
      }
      return this.model.set(this.attribute, point);
    };

    AxisHandler.prototype.showInputField = function() {
      return this.params.set({
        showInput: true
      });
    };

    AxisHandler.prototype.focusInputField = function() {
      return _.defer((function(_this) {
        return function() {
          var ref;
          return (ref = _this.subviews.inputControl) != null ? ref.$('input').focus() : void 0;
        };
      })(this));
    };

    AxisHandler.prototype.closeInputField = function() {
      return this.params.set({
        showInput: false
      });
    };

    AxisHandler.prototype.hasRecentlyOpenInput = function() {
      return this.params.get('showInput') || (new Date - this.lastFocusOutAt < 100);
    };

    AxisHandler.prototype.handleFocusOut = function() {
      this.lastFocusOutAt = new Date;
      return this.closeInputField();
    };

    AxisHandler.prototype.text = function(value) {
      return t(this.model.name + ".axisHandler." + this.attribute + ".label", this.model.name + ".axisHandler.label", 'axisHandler.label', {
        value: "<span></span>"
      });
    };

    AxisHandler.prototype.renderSpan = function() {
      var control, ref, ref1, text;
      if (this.params.get('showInput')) {
        this.$label.addClass('has-input');
        control = new Backbone.Poised.Textfield(_.defaults({
          model: this.model,
          attribute: this.attribute,
          type: 'number'
        }, this.formSettings));
        this.subviews.inputControl = control;
        this.listenToOnce(control, 'changeValue', this.handleFocusOut);
        return this.$label.find('span').html(control.render().el);
      } else {
        this.$label.removeClass('has-input');
        text = this.model.get(this.attribute).toFixed(this.precision);
        if (((ref = this.formSettings) != null ? (ref1 = ref.unit) != null ? ref1.length : void 0 : void 0) > 0) {
          text += ' ' + this.formSettings.unit;
        }
        return this.$label.find('span').text(text);
      }
    };

    AxisHandler.prototype.render = function() {
      delete this.subviews.inputField;
      this.$label = $('<label>').html(this.text());
      this.renderSpan();
      this.$el.html(this.$label);
      this.$el.addClass("axis-handler-" + this.position);
      return this;
    };

    return AxisHandler;

  })(Backbone.Poised.View);

}).call(this);
