(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  if (ELA.Views == null) {
    ELA.Views = {};
  }

  ELA.Views.BaseGraph = (function(superClass) {
    extend(BaseGraph, superClass);

    function BaseGraph() {
      this.render = bind(this.render, this);
      this.panend = bind(this.panend, this);
      this.pan = bind(this.pan, this);
      this.panstart = bind(this.panstart, this);
      this.pinchend = bind(this.pinchend, this);
      this.pinch = bind(this.pinch, this);
      this.pinchstart = bind(this.pinchstart, this);
      this.debugCircle = bind(this.debugCircle, this);
      this.linkedYScale = bind(this.linkedYScale, this);
      this.linkedXScale = bind(this.linkedXScale, this);
      this.resetCanvas = bind(this.resetCanvas, this);
      this.mousewheel = bind(this.mousewheel, this);
      this.maxRangeY = bind(this.maxRangeY, this);
      this.maxRangeX = bind(this.maxRangeX, this);
      this.calculateRanges = bind(this.calculateRanges, this);
      this.calculateRangeY = bind(this.calculateRangeY, this);
      this.calculateRangeX = bind(this.calculateRangeX, this);
      return BaseGraph.__super__.constructor.apply(this, arguments);
    }

    BaseGraph.Params = (function(superClass1) {
      extend(Params, superClass1);

      function Params() {
        return Params.__super__.constructor.apply(this, arguments);
      }

      Params.prototype.serialize = function() {
        return {
          xScale: this.get('xScale'),
          yScale: this.get('yScale'),
          xOriginRatio: this.get('xOrigin') / this.get('width'),
          yOriginRatio: this.get('yOrigin') / this.get('height')
        };
      };

      Params.prototype.deserialize = function(attributes) {
        return this.set({
          xScale: attributes.xScale,
          yScale: attributes.yScale,
          xOrigin: this.get('width') * attributes.xOriginRatio,
          yOrigin: this.get('height') * attributes.yOriginRatio
        });
      };

      return Params;

    })(Backbone.Model);

    BaseGraph.prototype.start = {};

    BaseGraph.prototype.params = null;

    BaseGraph.prototype.pixelRatio = null;

    BaseGraph.prototype.defaults = {
      xScale: 1.1,
      yScale: 1.1,
      xOrigin: 30,
      yOrigin: 0,
      xOriginAnchor: 'left',
      yOriginAnchor: 'bottom',
      xOriginRatio: null,
      yOriginRatio: null,
      yOffset: 0,
      debug: {},
      xLabelPosition: 'top',
      yLabelPosition: 'right',
      xPanLocked: false,
      yPanLocked: false,
      xPinchLocked: false,
      yPinchLocked: false,
      scaleLink: false
    };

    BaseGraph.prototype.initialize = function() {
      BaseGraph.__super__.initialize.apply(this, arguments);
      if ((this.defaults.xOriginRatio != null) && this.defaults.xOriginAnchor !== 'left') {
        this.defaults.xOriginRatio = 1 - this.defaults.xOriginRatio;
        this.defaults.xOriginAnchor = 'left';
      }
      if ((this.defaults.yOriginRatio != null) && this.defaults.yOriginAnchor === 'bottom') {
        this.defaults.yOriginRatio = 1 - this.defaults.yOriginRatio;
        this.defaults.yOriginAnchor = 'top';
      }
      this.resetCanvas();
      this.params.on('change:xScale', this.calculateRangeX);
      this.params.on('change:yScale', this.calculateRangeY);
      this.params.on('change:xOrigin change:yOrigin change:yRange change:xRange change:width change:height', this.requestRepaint);
      return this.calculateRanges();
    };

    BaseGraph.prototype.calculateRangeX = function() {
      return this.params.set({
        xRange: this.maxRangeX()
      });
    };

    BaseGraph.prototype.calculateRangeY = function() {
      return this.params.set({
        yRange: this.maxRangeY()
      });
    };

    BaseGraph.prototype.calculateRanges = function() {
      this.calculateRangeX();
      return this.calculateRangeY();
    };

    BaseGraph.prototype.maxRangeX = function(func, xScale) {
      if (xScale == null) {
        xScale = this.params.get('xScale');
      }
      return xScale * 5;
    };

    BaseGraph.prototype.maxRangeY = function(func, yScale) {
      if (yScale == null) {
        yScale = this.params.get('yScale');
      }
      return yScale * 5;
    };

    BaseGraph.prototype.events = {
      'pan': 'pan',
      'panstart': 'panstart',
      'panend': 'panend',
      'pinchstart': 'pinchstart',
      'pinch': 'pinch',
      'pinchend': 'pinchend',
      'doubletap': 'resetCanvas',
      'mousewheel': 'mousewheel',
      'DOMMouseScroll': 'mousewheel'
    };

    BaseGraph.prototype.hammerjs = {
      recognizers: [
        [
          Hammer.Rotate, {
            enable: false
          }
        ], [Hammer.Pinch, {}, ['rotate']], [
          Hammer.Swipe, {
            enable: false
          }
        ], [
          Hammer.Pan, {
            direction: Hammer.DIRECTION_ALL,
            threshold: 1
          }, ['swipe']
        ], [
          Hammer.Tap, {
            threshold: 5
          }
        ], [
          Hammer.Tap, {
            event: 'doubletap',
            taps: 2,
            posThreshold: 20,
            threshold: 5
          }, ['tap']
        ], [
          Hammer.Press, {
            enable: false
          }
        ]
      ]
    };

    BaseGraph.prototype.mousewheel = function(e) {
      var factor;
      e.preventDefault();
      if (e.originalEvent.wheelDelta > 0 || e.originalEvent.detail < 0) {
        factor = 1.05;
      } else {
        factor = 0.95;
      }
      if (!this.params.get('xPinchLocked') && (!window.keys.ALT || window.keys.SHIFT || this.params.get('scaleLink'))) {
        this.params.set({
          xScale: this.params.get('xScale') * factor
        });
      }
      if (!this.params.get('yPinchLocked') && (this.params.get('scaleLink') || window.keys.ALT || window.keys.SHIFT)) {
        return this.params.set({
          yScale: this.params.get('yScale') * factor
        });
      }
    };

    BaseGraph.prototype.resetCanvas = function() {
      var xOrigin, xScale, yOrigin, yScale;
      clearTimeout(this.panendTimeout);
      if (this.defaults.xOriginRatio != null) {
        xOrigin = this.params.get('width') * this.defaults.xOriginRatio;
      } else if (this.defaults.xOriginAnchor === 'left') {
        xOrigin = this.defaults.xOrigin;
      } else {
        xOrigin = this.params.get('width') - this.defaults.xOrigin;
      }
      if (this.defaults.yOriginRatio != null) {
        yOrigin = this.params.get('height') * this.defaults.yOriginRatio;
      } else if (this.defaults.yOriginAnchor === 'bottom') {
        yOrigin = this.params.get('height') - this.defaults.yOrigin;
      } else {
        yOrigin = this.defaults.yOrigin;
      }
      xScale = this.defaults.xScale;
      yScale = this.defaults.yScale;
      if (this.params.get('scaleLink')) {
        if (this.params.get('width') / this.maxRangeX(null, 1) < this.params.get('height') / this.maxRangeY(null, 1)) {
          yScale = this.linkedYScale(xScale);
        } else {
          xScale = this.linkedXScale(yScale);
        }
      }
      return this.params.set({
        xScale: xScale,
        yScale: yScale,
        xOrigin: xOrigin,
        yOrigin: yOrigin
      });
    };

    BaseGraph.prototype.linkedXScale = function(yScale) {
      return (this.params.get('width') / this.maxRangeX(null, 1)) / (this.params.get('height') / this.maxRangeY(null, yScale));
    };

    BaseGraph.prototype.linkedYScale = function(xScale) {
      return (this.params.get('height') / this.maxRangeY(null, 1)) / (this.params.get('width') / this.maxRangeX(null, xScale));
    };

    BaseGraph.prototype.debugCircle = function(context, color, x, y, radius) {
      if (radius == null) {
        radius = 20;
      }
      context.beginPath();
      context.arc(x, y, radius, 0, 2 * Math.PI, false);
      context.fillStyle = color;
      context.fill();
      return context.closePath();
    };

    BaseGraph.prototype.centerBetweenTouches = function(t1, t2) {
      return this.centerBetweenPoints(t1.clientX, t1.clientY, t2.clientX, t2.clientY);
    };

    BaseGraph.prototype.centerBetweenPoints = function(x1, y1, x2, y2) {
      var centerX, centerY;
      centerX = null;
      if (x1 <= x2) {
        centerX = x1 + (x2 - x1) / 2;
      } else {
        centerX = x2 + (x1 - x2) / 2;
      }
      centerY = null;
      if (y1 <= y2) {
        centerY = y1 + (y2 - y1) / 2;
      } else {
        centerY = y2 + (y1 - y2) / 2;
      }
      return {
        x: centerX,
        y: centerY
      };
    };

    BaseGraph.prototype.pinchstart = function(e) {
      var center;
      center = this.centerBetweenTouches(e.gesture.pointers[0], e.gesture.pointers[1]);
      this.start.centerX = center.x;
      this.start.centerY = center.y;
      this.start.xOrigin = this.params.get('xOrigin');
      this.start.yOrigin = this.params.get('yOrigin');
      this.start.touchDistanceX = Math.max(50, Math.abs(e.gesture.pointers[0].clientX - e.gesture.pointers[1].clientX));
      this.start.touchDistanceY = Math.max(50, Math.abs(e.gesture.pointers[0].clientY - e.gesture.pointers[1].clientY));
      this.start.xScale = this.params.get('xScale');
      return this.start.yScale = this.params.get('yScale');
    };

    BaseGraph.prototype.pinch = function(e) {
      var center, deltaX, deltaY, touchDistanceX, touchDistanceY, xScale, yScale;
      center = this.centerBetweenTouches(e.gesture.pointers[0], e.gesture.pointers[1]);
      deltaX = -(this.start.centerX - center.x);
      deltaY = -(this.start.centerY - center.y);
      touchDistanceX = Math.max(50, Math.abs(e.gesture.pointers[0].clientX - e.gesture.pointers[1].clientX));
      touchDistanceY = Math.max(50, Math.abs(e.gesture.pointers[0].clientY - e.gesture.pointers[1].clientY));
      xScale = touchDistanceX / this.start.touchDistanceX;
      yScale = touchDistanceY / this.start.touchDistanceY;
      if (this.params.get('scaleLink')) {
        xScale = yScale = (xScale + yScale) / 2;
      }
      if (!this.params.get('xPinchLocked')) {
        this.params.set('xScale', this.start.xScale / xScale);
      }
      if (!this.params.get('yPinchLocked')) {
        this.params.set('yScale', this.start.yScale / yScale);
      }
      if (!this.params.get('xPanLocked')) {
        this.params.set({
          xOrigin: this.start.centerX + (this.start.xOrigin - this.start.centerX) * xScale + deltaX
        });
      }
      if (!this.params.get('yPanLocked')) {
        return this.params.set({
          yOrigin: this.start.centerY + (this.start.yOrigin - this.start.centerY) * yScale + deltaY
        });
      }
    };

    BaseGraph.prototype.pinchend = function(e) {
      this.hammer.stop();
      return this.start = {};
    };

    BaseGraph.prototype.panstart = function() {
      clearTimeout(this.panendTimeout);
      this.start.xOrigin = this.params.get('xOrigin');
      return this.start.yOrigin = this.params.get('yOrigin');
    };

    BaseGraph.prototype.pan = function(e) {
      var base, base1;
      (base = this.start).xOrigin || (base.xOrigin = this.params.get('xOrigin'));
      (base1 = this.start).yOrigin || (base1.yOrigin = this.params.get('yOrigin'));
      if (!this.params.get('xPanLocked')) {
        this.params.set({
          xOrigin: this.start.xOrigin + e.gesture.deltaX
        });
      }
      if (!this.params.get('yPanLocked')) {
        return this.params.set({
          yOrigin: this.start.yOrigin + e.gesture.deltaY
        });
      }
    };

    BaseGraph.prototype.panend = function(e) {
      var slowdown, velocityX, velocityY;
      this.start = {};
      velocityX = e.gesture.velocityX * 10;
      velocityY = e.gesture.velocityY * 10;
      slowdown = (function(_this) {
        return function() {
          if (Math.abs(velocityX) > 0 && !_this.params.get('xPanLocked')) {
            _this.params.set({
              xOrigin: _this.params.get('xOrigin') + velocityX
            });
            velocityX *= 0.9;
          }
          if (Math.abs(velocityY) > 0 && !_this.params.get('yPanLocked')) {
            _this.params.set({
              yOrigin: _this.params.get('yOrigin') + velocityY
            });
            velocityY *= 0.9;
          }
          if (Math.abs(velocityY) > 0.2 || Math.abs(velocityX) > 0.2) {
            return _this.panendTimeout = setTimeout(slowdown, 8);
          }
        };
      })(this);
      return slowdown();
    };

    BaseGraph.prototype.roundStepSize = function(stepSize) {
      var breakpoint, denominator, factor, i, j, lastRoundedStepSize, len, len1, ref, ref1, roundedStepSize, validStepSize;
      validStepSize = (stepSize != null) && isFinite(stepSize);
      if (validStepSize && stepSize > 0.5) {
        factor = 1;
        while (true) {
          ref = [1, 2, 5];
          for (i = 0, len = ref.length; i < len; i++) {
            breakpoint = ref[i];
            roundedStepSize = breakpoint * factor;
            if (roundedStepSize > stepSize) {
              return roundedStepSize;
            }
          }
          factor *= 10;
        }
      } else if (validStepSize && (0 < stepSize && stepSize <= 0.5)) {
        denominator = 10;
        lastRoundedStepSize = null;
        while (true) {
          ref1 = [5, 2, 1];
          for (j = 0, len1 = ref1.length; j < len1; j++) {
            breakpoint = ref1[j];
            roundedStepSize = breakpoint / denominator;
            if (stepSize > roundedStepSize) {
              return lastRoundedStepSize || roundedStepSize;
            } else {
              lastRoundedStepSize = roundedStepSize;
            }
          }
          denominator *= 10;
        }
      } else {
        return stepSize;
      }
    };

    BaseGraph.prototype.axisLabel = function(value, stepSize) {
      var precision;
      precision = 0;
      while (stepSize && stepSize * Math.pow(10, precision) < 1) {
        precision += 1;
      }
      return value.toFixed(precision);
    };

    BaseGraph.prototype.setCanvasResolution = function(params) {
      var previous, previousXOrigin, previousYOrigin;
      BaseGraph.__super__.setCanvasResolution.apply(this, arguments);
      if (params != null) {
        previous = this.params.previousAttributes();
        previousXOrigin = previous.xOrigin != null ? previous.xOrigin : this.params.get('xOrigin');
        previousYOrigin = previous.yOrigin != null ? previous.yOrigin : this.params.get('yOrigin');
        this.params.set({
          xOrigin: previousXOrigin * this.params.get('width') / previous.width,
          yOrigin: previousYOrigin * this.params.get('height') / previous.height
        });
        switch (this.params.get('scaleLink')) {
          case 'x':
            return this.params.set({
              xScale: this.linkedXScale()
            });
          case 'y':
            return this.params.set({
              yScale: this.linkedYScale()
            });
        }
      }
    };

    BaseGraph.prototype.xAxisValueLabel = function(val, stepsize) {
      return this.axisLabel(val, stepsize);
    };

    BaseGraph.prototype.yAxisValueLabel = function(val, stepsize) {
      return this.axisLabel(val - this.params.get('yOffset'), stepsize);
    };

    BaseGraph.prototype._axisLabel = function(label) {
      if (typeof label === 'function') {
        return label.call(this);
      } else if (label != null) {
        return label;
      } else {
        return '';
      }
    };

    BaseGraph.prototype.xAxisLabel = function() {
      return this._axisLabel(this.params.get('xAxisLabel'));
    };

    BaseGraph.prototype.yAxisLabel = function() {
      return this._axisLabel(this.params.get('yAxisLabel'));
    };

    BaseGraph.prototype.renderCurves = function() {};

    BaseGraph.prototype.renderXAxis = function() {
      var dashY, i, labelY, ref, ref1, ref2, x, xMax, xMin, xPos, xRange, xStepSize;
      this.context.setLineDash([]);
      this.context.font = this.defaultFont;
      this.context.strokeStyle = "#999999";
      this.context.fillStyle = "#000000";
      this.context.lineWidth = 2;
      this.context.beginPath();
      this.context.moveTo(0, this.yOrigin);
      this.context.lineTo(this.width, this.yOrigin);
      if (this.params.get('xLabelPosition') !== 'none') {
        labelY = dashY = 0;
        if (this.params.get('xLabelPosition') === 'bottom') {
          labelY = this.yOrigin + 15;
          dashY = this.yOrigin + 5;
        } else {
          labelY = this.yOrigin - 15;
          dashY = this.yOrigin - 5;
        }
        this.context.textBaseline = 'middle';
        this.context.textAlign = 'center';
        if (typeof xRange === "undefined" || xRange === null) {
          xRange = this.maxRangeX();
        }
        xMin = -this.xOrigin * xRange / this.width;
        xMax = (this.width - this.xOrigin) * xRange / this.width;
        xStepSize = this.roundStepSize(100 * xRange / this.width);
        for (x = i = ref = xMin - xMin % xStepSize, ref1 = xMax, ref2 = xStepSize; ref2 > 0 ? i <= ref1 : i >= ref1; x = i += ref2) {
          if (x <= (0 - xStepSize / 2) || (0 + xStepSize / 2) <= x) {
            xPos = this.xOrigin + x * this.width / xRange;
            this.context.moveTo(xPos, this.yOrigin);
            this.context.lineTo(xPos, dashY);
            this.context.fillText("" + (this.xAxisValueLabel(x, xStepSize)), xPos, labelY);
          }
        }
      }
      this.context.save();
      xPos = this.xOrigin <= this.width / 2 ? (this.xOrigin + this.width) / 2 : this.xOrigin / 2;
      if (this.context.pixelRatio === 1) {
        xPos = Math.round(xPos);
      }
      this.context.translate(xPos, this.yOrigin + 15);
      MarkupText.render(this.context, this.xAxisLabel(), this.defaultFont);
      this.context.restore();
      this.context.stroke();
      return this.context.closePath();
    };

    BaseGraph.prototype.renderYAxis = function(yRange) {
      var dashX, i, labelX, ref, ref1, ref2, y, yMax, yMin, yPos, yStepSize;
      this.context.setLineDash([]);
      this.context.font = this.defaultFont;
      this.context.strokeStyle = "#999999";
      this.context.fillStyle = "#000000";
      this.context.lineWidth = 2;
      this.context.beginPath();
      this.context.moveTo(this.xOrigin, 0);
      this.context.lineTo(this.xOrigin, this.height);
      if (this.params.get('yLabelPosition') !== 'none') {
        labelX = dashX = 0;
        if (this.params.get('yLabelPosition') === 'right') {
          labelX = this.xOrigin + 10;
          dashX = this.xOrigin + 5;
          this.context.textAlign = 'left';
        } else {
          labelX = this.xOrigin - 10;
          dashX = this.xOrigin - 5;
          this.context.textAlign = 'right';
        }
        this.context.textBaseline = 'middle';
        if (yRange == null) {
          yRange = this.maxRangeY();
        }
        yMin = -(this.height - this.yOrigin) * yRange / this.height;
        yMax = this.yOrigin * yRange / this.height;
        yStepSize = this.roundStepSize(100 * yRange / this.height);
        for (y = i = ref = yMin - yMin % yStepSize, ref1 = yMax, ref2 = yStepSize; ref2 > 0 ? i <= ref1 : i >= ref1; y = i += ref2) {
          if (y <= (0 - yStepSize / 2) || y >= (0 + yStepSize / 2)) {
            yPos = this.yOrigin - y * this.height / yRange;
            this.context.moveTo(this.xOrigin, yPos);
            this.context.lineTo(dashX, yPos);
            this.context.fillText(this.yAxisValueLabel(y, yStepSize), labelX, yPos);
          }
        }
      }
      this.context.save();
      yPos = this.yOrigin < this.height / 2 ? (this.yOrigin + this.height) / 2 : this.yOrigin / 2;
      if (this.context.pixelRatio === 1) {
        yPos = Math.round(yPos);
      }
      this.context.translate(this.xOrigin - 15, yPos);
      this.context.rotate(-Math.PI / 2);
      MarkupText.render(this.context, this.yAxisLabel(), this.defaultFont);
      this.context.restore();
      this.context.stroke();
      return this.context.closePath();
    };

    BaseGraph.prototype.renderGrid = function() {
      var i, j, ref, ref1, ref2, ref3, ref4, ref5, x, xMax, xMin, xPos, xRange, xStepSize, y, yMax, yMin, yPos, yRange, yStepSize;
      this.context.strokeStyle = '#eeeeee';
      this.context.lineWidth = 1;
      this.context.beginPath();
      if (this.params.get('xLabelPosition') !== 'none') {
        if (typeof xRange === "undefined" || xRange === null) {
          xRange = this.maxRangeX();
        }
        xMin = -this.xOrigin * xRange / this.width;
        xMax = (this.width - this.xOrigin) * xRange / this.width;
        xStepSize = this.roundStepSize(100 * xRange / this.width);
        for (x = i = ref = xMin - xMin % xStepSize, ref1 = xMax, ref2 = xStepSize; ref2 > 0 ? i <= ref1 : i >= ref1; x = i += ref2) {
          xPos = this.xOrigin + x * this.width / xRange;
          this.context.moveTo(xPos, 0);
          this.context.lineTo(xPos, this.height);
        }
      }
      if (this.params.get('yLabelPosition') !== 'none') {
        if (typeof yRange === "undefined" || yRange === null) {
          yRange = this.maxRangeY();
        }
        yMin = -(this.height - this.yOrigin) * yRange / this.height;
        yMax = this.yOrigin * yRange / this.height;
        yStepSize = this.roundStepSize(100 * yRange / this.height);
        for (y = j = ref3 = yMin - yMin % yStepSize, ref4 = yMax, ref5 = yStepSize; ref5 > 0 ? j <= ref4 : j >= ref4; y = j += ref5) {
          yPos = this.yOrigin - y * this.height / yRange;
          this.context.moveTo(0, yPos);
          this.context.lineTo(this.width, yPos);
        }
      }
      this.context.stroke();
      return this.context.closePath();
    };

    BaseGraph.prototype.beforeRender = function() {};

    BaseGraph.prototype.render = function() {
      this.width = this.params.get('width');
      this.height = this.params.get('height');
      this.xOrigin = this.params.get('xOrigin');
      this.yOrigin = this.params.get('yOrigin');
      this.yOffset = this.params.get('yOffset');
      this.xRange = this.params.get('xRange');
      this.yRange = this.params.get('yRange');
      this.xMin = -this.xOrigin * this.xRange / this.width;
      this.xMax = (this.width - this.xOrigin) * this.xRange / this.width;
      this.yMin = -(this.height - this.yOrigin) * this.yRange / this.height;
      this.yMax = this.yOrigin * this.yRange / this.height;
      if (this.width === null && this.height === null) {
        return this;
      }
      this.context || (this.context = this.el.getContext('2d'));
      this.context.clearRect(0, 0, this.width, this.height);
      this.beforeRender();
      this.renderGrid();
      this.renderXAxis();
      this.renderYAxis();
      this.renderCurves();
      return this;
    };

    return BaseGraph;

  })(ELA.Views.Canvas);

}).call(this);
