(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  if (ELA.Views == null) {
    ELA.Views = {};
  }

  ELA.Views.CurveGraph = (function(superClass) {
    extend(CurveGraph, superClass);

    function CurveGraph() {
      this.render = bind(this.render, this);
      this.maxRangeY = bind(this.maxRangeY, this);
      this.maxRangeX = bind(this.maxRangeX, this);
      this.calculateRanges = bind(this.calculateRanges, this);
      return CurveGraph.__super__.constructor.apply(this, arguments);
    }

    CurveGraph.prototype.initialize = function(options) {
      if (options == null) {
        options = {};
      }
      CurveGraph.__super__.initialize.apply(this, arguments);
      this.bindCalculatorEvents();
      this.model.curves.on('change:selected', (function(_this) {
        return function() {
          _this.bindCalculatorEvents();
          return _this.requestRepaint();
        };
      })(this));
      this.model.on('change:calculators', (function(_this) {
        return function() {
          _this.calculateRanges();
          _this.bindCalculatorEvents();
          return _this.requestRepaint();
        };
      })(this));
      this.model.on('change:valueAtRange', this.requestRepaint);
      return this.model.on('change:axisLabelingForCurve', this.requestRepaint);
    };

    CurveGraph.prototype.bindCalculatorEvents = function() {
      var calc, curve, k, l, len, len1, oldestCurve, ref, ref1, results;
      this.stopListening(this.model.previous('calculators'));
      ref = this.model.get('calculators');
      results = [];
      for (k = 0, len = ref.length; k < len; k++) {
        calc = ref[k];
        ref1 = this.model.curves.history;
        for (l = 0, len1 = ref1.length; l < len1; l++) {
          curve = ref1[l];
          this.listenTo(calc, "change:" + (curve.get('function')), this.requestRepaint);
        }
        this.listenTo(calc, 'change:maxX change:maxY', this.calculateRanges);
        oldestCurve = this.model.curves.history[0];
        if (oldestCurve != null) {
          results.push(this.listenTo(calc, "change:" + (oldestCurve.get('function')), this.calculateRanges));
        } else {
          results.push(void 0);
        }
      }
      return results;
    };

    CurveGraph.prototype.calculateRanges = function() {
      return this.params.set({
        xRange: this.maxRangeX()
      });
    };

    CurveGraph.prototype.maxRangeX = function() {
      var max, min, range;
      max = _.chain(this.model.get('calculators')).map(function(c) {
        return c.maxX();
      }).max().value();
      min = _.chain(this.model.get('calculators')).map(function(c) {
        return c.minX();
      }).min().value();
      range = 0;
      if (max > 0) {
        range += max;
      }
      if (min < 0) {
        range -= min;
      }
      return (Math.abs(range) || 10000) * this.params.get('xScale') * 1.1;
    };

    CurveGraph.prototype.maxRangeY = function(func) {
      var max, min, range, ref;
      if (func == null) {
        func = (ref = this.model.get('axisLabelingForCurve')) != null ? ref.get('function') : void 0;
      }
      max = _.chain(this.model.get('calculators')).map(function(c) {
        return c.maxY(func, c.minX(), c.maxX(), 30);
      }).max().value();
      min = _.chain(this.model.get('calculators')).map(function(c) {
        return c.minY(func, c.minX(), c.maxX(), 30);
      }).min().value();
      range = 0;
      if (max > 0) {
        range += max;
      }
      if (min < 0) {
        range -= min;
      }
      return Math.abs(range) * this.params.get('yScale') * 1.1;
    };

    CurveGraph.prototype.xAxisValueLabel = function(val, stepsize) {
      return this.axisLabel(val, stepsize);
    };

    CurveGraph.prototype.yAxisValueLabel = function(val, stepsize) {
      var curve;
      curve = this.model.get('axisLabelingForCurve');
      if (curve != null) {
        val = this.Present(curve).unitValue(val);
      }
      return this.axisLabel(val, stepsize);
    };

    CurveGraph.prototype.yAxisLabel = function() {
      var curve;
      curve = this.model.get('axisLabelingForCurve');
      return this.Present(curve).fullYAxisLabel();
    };

    CurveGraph.prototype.renderCurves = function() {
      var brokenLine, calc, curve, func, i, j, k, len, plotted, ref, results, val, x, xPos, y, yMax, yMin, yPos, yRange;
      ref = this.model.get('calculators');
      results = [];
      for (i = k = 0, len = ref.length; k < len; i = ++k) {
        calc = ref[i];
        results.push((function() {
          var l, len1, m, ref1, ref2, results1;
          ref1 = this.model.curves.history;
          results1 = [];
          for (j = l = 0, len1 = ref1.length; l < len1; j = ++l) {
            curve = ref1[j];
            func = curve.get('function');
            this.context.strokeStyle = curve.strokeStyle();
            this.context.lineWidth = 3;
            if (i === 1) {
              this.context.setLineDash([4, 5]);
            } else {
              this.context.setLineDash([]);
            }
            xPos = 0;
            brokenLine = 1;
            plotted = 0;
            this.context.beginPath();
            xPos = 0;
            yRange = this.maxRangeY(func);
            yMin = -(this.height - this.yOrigin) * yRange / this.height;
            yMax = this.yOrigin * yRange / this.height;
            for (xPos = m = -2, ref2 = this.width + 3; -2 <= ref2 ? m <= ref2 : m >= ref2; xPos = -2 <= ref2 ? ++m : --m) {
              x = this.xMin + xPos * this.xRange / this.width;
              val = calc[func](x);
              y = val;
              if (y != null) {
                yPos = (yMax - y) * this.height / yRange;
                if (brokenLine > 0) {
                  if (brokenLine === 1) {
                    this.context.moveTo(xPos, yPos);
                  }
                  brokenLine -= 1;
                } else {
                  this.context.lineTo(xPos, yPos);
                  plotted += 1;
                }
              }
            }
            this.context.stroke();
            results1.push(this.context.closePath());
          }
          return results1;
        }).call(this));
      }
      return results;
    };

    CurveGraph.prototype.renderRangeIndicator = function() {
      var valueAtPoint, xPos;
      this.context.setLineDash([]);
      this.context.beginPath();
      this.context.strokeStyle = '#999999';
      this.context.lineWidth = 2;
      this.context.moveTo(0, 0);
      this.context.lineTo(this.width, 0);
      this.context.stroke();
      this.context.beginPath();
      this.context.strokeStyle = '#999999';
      this.context.lineWidth = 2;
      this.context.moveTo(0, this.height);
      this.context.lineTo(this.width, this.height);
      this.context.stroke();
      if (this.model.get('valueAtRange') != null) {
        valueAtPoint = this.model.get('valueAtRange');
        this.context.beginPath();
        this.context.strokeStyle = '#999999';
        this.context.lineWidth = 2;
        xPos = this.xOrigin + valueAtPoint * this.width / this.xRange;
        this.context.moveTo(xPos, 0);
        this.context.lineTo(xPos, this.height);
        this.context.stroke();
        this.context.closePath();
        this.context.beginPath();
        this.context.lineWidth = 1;
        this.context.fillStyle = "#f2f2f2";
        this.context.moveTo(xPos + 8, 0);
        this.context.lineTo(xPos, 8);
        this.context.lineTo(xPos - 8, 0);
        this.context.moveTo(xPos - 8, this.height);
        this.context.lineTo(xPos, this.height - 8);
        this.context.lineTo(xPos + 8, this.height);
        this.context.stroke();
        this.context.fill();
        return this.context.closePath();
      }
    };

    CurveGraph.prototype.render = function() {
      CurveGraph.__super__.render.apply(this, arguments);
      this.renderRangeIndicator();
      return this;
    };

    return CurveGraph;

  })(ELA.Views.BaseGraph);

}).call(this);
