(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  if (ELA.Views == null) {
    ELA.Views = {};
  }

  ELA.Views.Graph = (function(superClass) {
    extend(Graph, superClass);

    function Graph() {
      this.render = bind(this.render, this);
      this.panend = bind(this.panend, this);
      this.pan = bind(this.pan, this);
      this.panstart = bind(this.panstart, this);
      this.pinchend = bind(this.pinchend, this);
      this.pinch = bind(this.pinch, this);
      this.pinchstart = bind(this.pinchstart, this);
      this.debugCircle = bind(this.debugCircle, this);
      this.resetCanvas = bind(this.resetCanvas, this);
      this.mousewheel = bind(this.mousewheel, this);
      return Graph.__super__.constructor.apply(this, arguments);
    }

    Graph.Params = ELA.Models.GraphParams;

    Graph.prototype.start = {};

    Graph.prototype.params = null;

    Graph.prototype.pixelRatio = null;

    Graph.prototype.initialize = function() {
      var guide, k, len, ref, results;
      Graph.__super__.initialize.apply(this, arguments);
      this.listenTo(this.params, 'change:xOrigin change:yOrigin ' + 'change:yRange change:xRange ' + 'change:width change:height ' + 'change:axisLabelingForCurve', this.requestRepaint);
      this.listenTo(this.model.curves, 'change:selected', this.requestRepaint);
      this.listenTo(this.model, 'change:calculators', this.requestRepaint);
      ref = this.params.get('guides');
      results = [];
      for (k = 0, len = ref.length; k < len; k++) {
        guide = ref[k];
        results.push(this.listenTo(this.model, "change:" + guide.attribute, this.requestRepaint));
      }
      return results;
    };

    Graph.prototype.events = {
      pan: 'pan',
      panstart: 'panstart',
      panend: 'panend',
      pinchstart: 'pinchstart',
      pinch: 'pinch',
      pinchend: 'pinchend',
      doubletap: 'resetCanvas',
      mousewheel: 'mousewheel',
      DOMMouseScroll: 'mousewheel'
    };

    Graph.prototype.hammerjs = {
      recognizers: [
        [
          Hammer.Rotate, {
            enable: false
          }
        ], [Hammer.Pinch, {}, ['rotate']], [
          Hammer.Swipe, {
            enable: false
          }
        ], [
          Hammer.Pan, {
            direction: Hammer.DIRECTION_ALL,
            threshold: 1
          }, ['swipe']
        ], [
          Hammer.Tap, {
            threshold: 5
          }
        ], [
          Hammer.Tap, {
            event: 'doubletap',
            taps: 2,
            posThreshold: 20,
            threshold: 5
          }, ['tap']
        ], [
          Hammer.Press, {
            enable: false
          }
        ]
      ]
    };

    Graph.prototype.mousewheel = function(e) {
      var factor;
      e.preventDefault();
      if (e.originalEvent.wheelDelta > 0 || e.originalEvent.detail < 0) {
        factor = 1.05;
      } else {
        factor = 0.95;
      }
      if (!this.params.get('xPinchLocked') && (!window.keys.ALT || window.keys.SHIFT || this.params.get('scaleLink'))) {
        this.params.set({
          xScale: this.params.get('xScale') * factor
        });
      }
      if (!this.params.get('yPinchLocked') && (this.params.get('scaleLink') || window.keys.ALT || window.keys.SHIFT)) {
        return this.params.set({
          yScale: this.params.get('yScale') * factor
        });
      }
    };

    Graph.prototype.resetCanvas = function() {
      clearTimeout(this.panendTimeout);
      return this.params.reset();
    };

    Graph.prototype.debugCircle = function(context, color, x, y, radius) {
      if (radius == null) {
        radius = 20;
      }
      context.beginPath();
      context.arc(x, y, radius, 0, 2 * Math.PI, false);
      context.fillStyle = color;
      context.fill();
      return context.closePath();
    };

    Graph.prototype.centerBetweenTouches = function(t1, t2) {
      return this.centerBetweenPoints(t1.clientX, t1.clientY, t2.clientX, t2.clientY);
    };

    Graph.prototype.centerBetweenPoints = function(x1, y1, x2, y2) {
      var centerX, centerY;
      centerX = null;
      if (x1 <= x2) {
        centerX = x1 + (x2 - x1) / 2;
      } else {
        centerX = x2 + (x1 - x2) / 2;
      }
      centerY = null;
      if (y1 <= y2) {
        centerY = y1 + (y2 - y1) / 2;
      } else {
        centerY = y2 + (y1 - y2) / 2;
      }
      return {
        x: centerX,
        y: centerY
      };
    };

    Graph.prototype.pinchstart = function(e) {
      var center;
      center = this.centerBetweenTouches(e.gesture.pointers[0], e.gesture.pointers[1]);
      this.start.centerX = center.x;
      this.start.centerY = center.y;
      this.start.xOrigin = this.params.get('xOrigin');
      this.start.yOrigin = this.params.get('yOrigin');
      this.start.touchDistanceX = Math.max(50, Math.abs(e.gesture.pointers[0].clientX - e.gesture.pointers[1].clientX));
      this.start.touchDistanceY = Math.max(50, Math.abs(e.gesture.pointers[0].clientY - e.gesture.pointers[1].clientY));
      this.start.xScale = this.params.get('xScale');
      return this.start.yScale = this.params.get('yScale');
    };

    Graph.prototype.pinch = function(e) {
      var center, deltaX, deltaY, touchDistanceX, touchDistanceY, xScale, yScale;
      center = this.centerBetweenTouches(e.gesture.pointers[0], e.gesture.pointers[1]);
      deltaX = -(this.start.centerX - center.x);
      deltaY = -(this.start.centerY - center.y);
      touchDistanceX = Math.max(50, Math.abs(e.gesture.pointers[0].clientX - e.gesture.pointers[1].clientX));
      touchDistanceY = Math.max(50, Math.abs(e.gesture.pointers[0].clientY - e.gesture.pointers[1].clientY));
      xScale = touchDistanceX / this.start.touchDistanceX;
      yScale = touchDistanceY / this.start.touchDistanceY;
      if (this.params.get('scaleLink')) {
        xScale = yScale = (xScale + yScale) / 2;
      }
      if (!this.params.get('xPinchLocked')) {
        this.params.set('xScale', this.start.xScale / xScale);
      }
      if (!this.params.get('yPinchLocked')) {
        this.params.set('yScale', this.start.yScale / yScale);
      }
      if (!this.params.get('xPanLocked')) {
        this.params.set({
          xOrigin: this.start.centerX + (this.start.xOrigin - this.start.centerX) * xScale + deltaX
        });
      }
      if (!this.params.get('yPanLocked')) {
        return this.params.set({
          yOrigin: this.start.centerY + (this.start.yOrigin - this.start.centerY) * yScale + deltaY
        });
      }
    };

    Graph.prototype.pinchend = function(e) {
      this.hammer.stop();
      return this.start = {};
    };

    Graph.prototype.panstart = function() {
      clearTimeout(this.panendTimeout);
      this.start.xOrigin = this.params.get('xOrigin');
      return this.start.yOrigin = this.params.get('yOrigin');
    };

    Graph.prototype.pan = function(e) {
      var base, base1;
      (base = this.start).xOrigin || (base.xOrigin = this.params.get('xOrigin'));
      (base1 = this.start).yOrigin || (base1.yOrigin = this.params.get('yOrigin'));
      if (!this.params.get('xPanLocked')) {
        this.params.set({
          xOrigin: this.start.xOrigin + e.gesture.deltaX
        });
      }
      if (!this.params.get('yPanLocked')) {
        return this.params.set({
          yOrigin: this.start.yOrigin + e.gesture.deltaY
        });
      }
    };

    Graph.prototype.panend = function(e) {
      var slowdown, velocityX, velocityY;
      this.start = {};
      velocityX = e.gesture.velocityX * 10;
      velocityY = e.gesture.velocityY * 10;
      slowdown = (function(_this) {
        return function() {
          if (Math.abs(velocityX) > 0 && !_this.params.get('xPanLocked')) {
            _this.params.set({
              xOrigin: _this.params.get('xOrigin') + velocityX
            });
            velocityX *= 0.9;
          }
          if (Math.abs(velocityY) > 0 && !_this.params.get('yPanLocked')) {
            _this.params.set({
              yOrigin: _this.params.get('yOrigin') + velocityY
            });
            velocityY *= 0.9;
          }
          if (Math.abs(velocityY) > 0.2 || Math.abs(velocityX) > 0.2) {
            return _this.panendTimeout = setTimeout(slowdown, 8);
          }
        };
      })(this);
      return slowdown();
    };

    Graph.prototype.roundStepSize = function(stepSize) {
      var breakpoint, denominator, factor, k, l, lastRoundedStepSize, len, len1, ref, ref1, roundedStepSize, validStepSize;
      validStepSize = (stepSize != null) && isFinite(stepSize);
      if (validStepSize && stepSize > 0.5) {
        factor = 1;
        while (true) {
          ref = [1, 2, 5];
          for (k = 0, len = ref.length; k < len; k++) {
            breakpoint = ref[k];
            roundedStepSize = breakpoint * factor;
            if (roundedStepSize > stepSize) {
              return roundedStepSize;
            }
          }
          factor *= 10;
        }
      } else if (validStepSize && (0 < stepSize && stepSize <= 0.5)) {
        denominator = 10;
        lastRoundedStepSize = null;
        while (true) {
          ref1 = [5, 2, 1];
          for (l = 0, len1 = ref1.length; l < len1; l++) {
            breakpoint = ref1[l];
            roundedStepSize = breakpoint / denominator;
            if (stepSize > roundedStepSize) {
              return lastRoundedStepSize || roundedStepSize;
            } else {
              lastRoundedStepSize = roundedStepSize;
            }
          }
          denominator *= 10;
        }
      } else {
        return stepSize;
      }
    };

    Graph.prototype.axisLabel = function(value, stepSize) {
      var precision;
      precision = 0;
      while (stepSize && stepSize * Math.pow(10, precision) < 1) {
        precision += 1;
      }
      return value.toFixed(precision);
    };

    Graph.prototype.xAxisValueLabel = function(val, stepsize) {
      var str;
      str = this.axisLabel(val, stepsize);
      switch (this.xAxisScale.type) {
        case 'linear':
          return str;
        case 'logarithmic':
          if (val === 1) {
            return this.xAxisScale.base;
          } else {
            return this.xAxisScale.base + "^" + str;
          }
      }
    };

    Graph.prototype.yAxisValueLabel = function(val, stepsize) {
      var curve, curvePresenter;
      curve = this.params.get('axisLabelingForCurve');
      if (curve != null) {
        curvePresenter = this.Present(curve);
        val = curvePresenter.unitValue(val);
        stepsize = curvePresenter.unitValue(stepsize);
      }
      return this.axisLabel(val - this.params.get('yOffset'), stepsize);
    };

    Graph.prototype.genericAxisLabel = function(axis) {
      var axisLabel, axisLabelLocale, curve;
      axisLabel = this.params.get(axis + "AxisLabel");
      if ((axisLabel != null) && _.isFunction(axisLabel)) {
        return axisLabel.call(this);
      }
      if (axisLabel != null) {
        return axisLabel;
      }
      axisLabelLocale = this.params.get(axis + "AxisLabelLocale");
      if (axisLabelLocale != null) {
        return this.loadLocale(axisLabelLocale);
      }
      curve = this.params.get('axisLabelingForCurve');
      if (curve != null) {
        if (axis === 'x') {
          return this.Present(curve).fullXAxisLabel();
        }
        if (axis === 'y') {
          return this.Present(curve).fullYAxisLabel();
        }
      }
      return this.loadLocale("graph." + axis + "AxisLabel", {
        defaultValue: ''
      });
    };

    Graph.prototype.xAxisLabel = function() {
      return this.genericAxisLabel('x');
    };

    Graph.prototype.yAxisLabel = function() {
      return this.genericAxisLabel('y');
    };

    Graph.prototype.beforeRenderCurves = function() {};

    Graph.prototype.afterRenderCurves = function() {};

    Graph.prototype.beforeRender = function() {};

    Graph.prototype.renderXAxis = function(y) {
      var dashY, k, labelY, ref, ref1, ref2, x, xMax, xMin, xPos, xRange, xStepSize, yPos;
      this.context.setLineDash([]);
      this.context.font = this.defaultFont;
      this.context.strokeStyle = "#999999";
      this.context.fillStyle = "#000000";
      this.context.lineWidth = 2;
      this.context.beginPath();
      yPos = this.yOrigin - y * this.height / this.yRange;
      this.context.moveTo(0, yPos);
      this.context.lineTo(this.width, yPos);
      if (this.params.get('xLabelPosition') !== 'none') {
        labelY = dashY = 0;
        if (this.params.get('xLabelPosition') === 'bottom') {
          labelY = yPos + 15;
          dashY = yPos + 5;
        } else {
          labelY = yPos - 15;
          dashY = yPos - 5;
        }
        this.context.textBaseline = 'middle';
        this.context.textAlign = 'center';
        xRange = this.params.maxRangeX();
        xMin = -this.xOrigin * xRange / this.width;
        xMax = (this.width - this.xOrigin) * xRange / this.width;
        xStepSize = this.roundStepSize(100 * xRange / this.width);
        for (x = k = ref = xMin - xMin % xStepSize, ref1 = xMax, ref2 = xStepSize; ref2 > 0 ? k <= ref1 : k >= ref1; x = k += ref2) {
          if (x <= (0 - xStepSize / 2) || (0 + xStepSize / 2) <= x) {
            xPos = this.xOrigin + x * this.width / xRange;
            this.context.moveTo(xPos, yPos);
            this.context.lineTo(xPos, dashY);
            this.context.fillText("" + (this.xAxisValueLabel(x, xStepSize)), xPos, labelY);
          }
        }
      }
      this.context.save();
      xPos = this.xOrigin <= this.width / 2 ? (this.xOrigin + this.width) / 2 : this.xOrigin / 2;
      if (this.context.pixelRatio === 1) {
        xPos = Math.round(xPos);
      }
      this.context.translate(xPos, yPos + 15);
      MarkupText.render(this.context, this.xAxisLabel(), this.defaultFont);
      this.context.restore();
      this.context.stroke();
      return this.context.closePath();
    };

    Graph.prototype.renderYAxis = function(x) {
      var dashX, k, labelX, ref, ref1, ref2, xPos, y, yMax, yMin, yPos, yRange, yStepSize;
      this.context.setLineDash([]);
      this.context.font = this.defaultFont;
      this.context.strokeStyle = "#999999";
      this.context.fillStyle = "#000000";
      this.context.lineWidth = 2;
      this.context.beginPath();
      xPos = this.xOrigin + x * this.width / this.xRange;
      this.context.moveTo(xPos, 0);
      this.context.lineTo(xPos, this.height);
      if (this.params.get('yLabelPosition') !== 'none') {
        labelX = dashX = 0;
        if (this.params.get('yLabelPosition') === 'right') {
          labelX = xPos + 10;
          dashX = xPos + 5;
          this.context.textAlign = 'left';
        } else {
          labelX = xPos - 10;
          dashX = xPos - 5;
          this.context.textAlign = 'right';
        }
        this.context.textBaseline = 'middle';
        yRange = this.params.maxRangeY();
        yMin = -(this.height - this.yOrigin) * yRange / this.height;
        yMax = this.yOrigin * yRange / this.height;
        yStepSize = this.roundStepSize(100 * yRange / this.height);
        for (y = k = ref = yMin - yMin % yStepSize, ref1 = yMax, ref2 = yStepSize; ref2 > 0 ? k <= ref1 : k >= ref1; y = k += ref2) {
          if (y <= (0 - yStepSize / 2) || y >= (0 + yStepSize / 2)) {
            yPos = this.yOrigin - y * this.height / yRange;
            this.context.moveTo(xPos, yPos);
            this.context.lineTo(dashX, yPos);
            this.context.fillText(this.yAxisValueLabel(y, yStepSize), labelX, yPos);
          }
        }
      }
      this.context.save();
      yPos = this.yOrigin < this.height / 2 ? (this.yOrigin + this.height) / 2 : this.yOrigin / 2;
      if (this.context.pixelRatio === 1) {
        yPos = Math.round(yPos);
      }
      this.context.translate(xPos - 15, yPos);
      this.context.rotate(-Math.PI / 2);
      MarkupText.render(this.context, this.yAxisLabel(), this.defaultFont);
      this.context.restore();
      this.context.stroke();
      return this.context.closePath();
    };

    Graph.prototype.renderGrid = function() {
      var k, l, ref, ref1, ref2, ref3, ref4, ref5, x, xMax, xMin, xPos, xRange, xStepSize, y, yMax, yMin, yPos, yRange, yStepSize;
      this.context.strokeStyle = '#eeeeee';
      this.context.lineWidth = 1;
      this.context.beginPath();
      if (this.params.get('xLabelPosition') !== 'none') {
        if (typeof xRange === "undefined" || xRange === null) {
          xRange = this.params.maxRangeX();
        }
        xMin = -this.xOrigin * xRange / this.width;
        xMax = (this.width - this.xOrigin) * xRange / this.width;
        xStepSize = this.roundStepSize(100 * xRange / this.width);
        for (x = k = ref = xMin - xMin % xStepSize, ref1 = xMax, ref2 = xStepSize; ref2 > 0 ? k <= ref1 : k >= ref1; x = k += ref2) {
          xPos = this.xOrigin + x * this.width / xRange;
          this.context.moveTo(xPos, 0);
          this.context.lineTo(xPos, this.height);
        }
      }
      if (this.params.get('yLabelPosition') !== 'none') {
        if (typeof yRange === "undefined" || yRange === null) {
          yRange = this.params.maxRangeY();
        }
        yMin = -(this.height - this.yOrigin) * yRange / this.height;
        yMax = this.yOrigin * yRange / this.height;
        yStepSize = this.roundStepSize(100 * yRange / this.height);
        for (y = l = ref3 = yMin - yMin % yStepSize, ref4 = yMax, ref5 = yStepSize; ref5 > 0 ? l <= ref4 : l >= ref4; y = l += ref5) {
          yPos = this.yOrigin - y * this.height / yRange;
          this.context.moveTo(0, yPos);
          this.context.lineTo(this.width, yPos);
        }
      }
      this.context.stroke();
      return this.context.closePath();
    };

    Graph.prototype.getControlPoints = function(x0, y0, x1, y1, x2, y2, tension) {
      var d01, d12, fa, fb, p1x, p1y, p2x, p2y;
      d01 = Math.sqrt(Math.pow(x1 - x0, 2) + Math.pow(y1 - y0, 2));
      d12 = Math.sqrt(Math.pow(x2 - x1, 2) + Math.pow(y2 - y1, 2));
      fa = tension * d01 / (d01 + d12);
      fb = tension - fa;
      p1x = x1 + fa * (x0 - x2);
      p1y = y1 + fa * (y0 - y2);
      p2x = x1 - fb * (x0 - x2);
      p2y = y1 - fb * (y0 - y2);
      return [p1x, p1y, p2x, p2y];
    };

    Graph.prototype.drawSpline = function(points, t, closed, xPos, yPos) {
      var cp, i, k, l, m, n, o, ref, ref1, ref2, ref3, results;
      if (xPos == null) {
        xPos = (function(x) {
          return x;
        });
      }
      if (yPos == null) {
        yPos = (function(y) {
          return y;
        });
      }
      if (points.length <= 4) {
        this.context.moveTo(xPos(points[0]), yPos(points[1]));
        return this.context.lineTo(xPos(points[2]), yPos(points[3]));
      } else {
        cp = [];
        n = points.length;
        points = _.clone(points);
        if (closed) {
          points.push(points[0], points[1], points[2], points[3]);
          points.unshift(points[n - 1]);
          points.unshift(points[n - 1]);
          for (i = k = 0, ref = n; k <= ref; i = k += 2) {
            cp = cp.concat(this.getControlPoints(points[i], points[i + 1], points[i + 2], points[i + 3], points[i + 4], points[i + 5], t));
          }
          cp = cp.concat(cp[0], cp[1]);
          results = [];
          for (i = l = 0, ref1 = n; l <= ref1; i = l += 2) {
            this.context.moveTo(xPos(points[i]), yPos(points[i + 1]));
            results.push(this.context.bezierCurveTo(xPos(cp[2 * i - 2]), yPos(cp[2 * i - 1]), xPos(cp[2 * i]), yPos(cp[2 * i + 1]), xPos(points[i + 2]), yPos(points[i + 3])));
          }
          return results;
        } else {
          for (i = m = 0, ref2 = n - 4; m <= ref2; i = m += 2) {
            cp = cp.concat(this.getControlPoints(points[i], points[i + 1], points[i + 2], points[i + 3], points[i + 4], points[i + 5], t));
          }
          this.context.moveTo(xPos(points[0]), yPos(points[1]));
          this.context.quadraticCurveTo(xPos(cp[0]), yPos(cp[1]), xPos(points[2]), yPos(points[3]));
          for (i = o = 2, ref3 = n - 3; o <= ref3; i = o += 2) {
            this.context.moveTo(xPos(points[i]), yPos(points[i + 1]));
            this.context.bezierCurveTo(xPos(cp[2 * i - 2]), yPos(cp[2 * i - 1]), xPos(cp[2 * i]), yPos(cp[2 * i + 1]), xPos(points[i + 2]), yPos(points[i + 3]));
          }
          this.context.moveTo(xPos(points[n - 2]), yPos(points[n - 1]));
          return this.context.quadraticCurveTo(xPos(cp[2 * n - 10]), yPos(cp[2 * n - 9]), xPos(points[n - 4]), yPos(points[n - 3]));
        }
      }
    };

    Graph.prototype.drawCurve = function(points, tension, closed, xPos, yPos) {
      var beginning, i, k, ref, results;
      if (tension == null) {
        tension = 0;
      }
      if (closed == null) {
        closed = false;
      }
      if (xPos == null) {
        xPos = (function(x) {
          return x;
        });
      }
      if (yPos == null) {
        yPos = (function(y) {
          return y;
        });
      }
      if (tension === 0) {
        beginning = true;
        if (points.length >= 2) {
          this.context.moveTo(xPos(points[0]), yPos(points[1]));
          results = [];
          for (i = k = 2, ref = points.length; k < ref; i = k += 2) {
            results.push(this.context.lineTo(xPos(points[i]), yPos(points[i + 1])));
          }
          return results;
        }
      } else {
        return this.drawSpline(points, tension, closed, xPos, yPos);
      }
    };

    Graph.prototype.renderCurves = function() {
      var calc, curve, func, i, j, k, l, len, len1, len2, m, name, ref, ref1, ref2, result, resultCurve, resultCurves, subcurve, xPos, xRange, yPos, yRange;
      resultCurves = [];
      ref = this.model.get('calculators');
      for (i = k = 0, len = ref.length; k < len; i = ++k) {
        calc = ref[i];
        ref1 = this.params.sortedCurves();
        for (j = l = 0, len1 = ref1.length; l < len1; j = ++l) {
          curve = ref1[j];
          func = curve.get('function');
          resultCurves.push({
            curve: curve,
            result: calc[func](this.xMin, this.xMax),
            yRange: this.params.maxRangeY(func),
            xRange: this.params.maxRangeX(func),
            calculatorIdx: i
          });
        }
      }
      this.beforeRenderCurves(resultCurves);
      for (m = 0, len2 = resultCurves.length; m < len2; m++) {
        resultCurve = resultCurves[m];
        curve = resultCurve.curve;
        result = resultCurve.result;
        yRange = resultCurve.yRange;
        xRange = resultCurve.xRange;
        if (_.isObject(result)) {
          xPos = (function(_this) {
            return function(x) {
              return _this.xOrigin + x * _this.width / xRange;
            };
          })(this);
          yPos = (function(_this) {
            return function(y) {
              return _this.yOrigin - (y + _this.yOffset) * _this.height / yRange;
            };
          })(this);
          if (curve.hasSubcurves()) {
            ref2 = curve.subcurves();
            for (name in ref2) {
              subcurve = ref2[name];
              if (result[name].points != null) {
                this.renderCurve(subcurve, result[name], xPos, yPos, resultCurve.calculatorIdx);
              } else if (result[name].radius != null) {
                this.renderCircle(subcurve, result[name], xPos, yPos, resultCurve.calculatorIdx);
              }
            }
          } else {
            if (result.points != null) {
              this.renderCurve(curve, result, xPos, yPos, resultCurve.calculatorIdx);
            } else if (result.radius != null) {
              this.renderCircle(curve, result, xPos, yPos, resultCurve.calculatorIdx);
            }
          }
        }
      }
      return this.afterRenderCurves(resultCurves);
    };

    Graph.prototype.renderCircle = function(curve, result, xPos, yPos, calculatorIdx) {
      var aX, aY, centerX, centerY, eX, eY, fillStyle, func, hB, height, mX, mY, vB, width, yRange;
      func = curve.get('function');
      yRange = this.maxRangeY(func);
      this.context.beginPath();
      width = Math.abs(xPos(2 * result.radius) - this.xOrigin);
      height = Math.abs(yPos(2 * result.radius) - this.yOrigin);
      centerX = xPos(result.center[0]);
      centerY = yPos(result.center[1]);
      aX = centerX - width / 2;
      aY = centerY - height / 2;
      hB = (width / 2) * .5522848;
      vB = (height / 2) * .5522848;
      eX = aX + width;
      eY = aY + height;
      mX = aX + width / 2;
      mY = aY + height / 2;
      this.context.moveTo(aX, mY);
      this.context.bezierCurveTo(aX, mY - vB, mX - hB, aY, mX, aY);
      this.context.bezierCurveTo(mX + hB, aY, eX, mY - vB, eX, mY);
      this.context.bezierCurveTo(eX, mY + vB, mX + hB, eY, mX, eY);
      this.context.bezierCurveTo(mX - hB, eY, aX, mY + vB, aX, mY);
      this.context.setLineDash(curve.lineDash(calculatorIdx));
      if (curve.strokeStyle()) {
        this.context.lineWidth = curve.get('lineWidth');
        this.context.strokeStyle = curve.strokeStyle();
        this.context.stroke();
      }
      if (fillStyle = curve.get('fillStyle')) {
        this.context.fillStyle = fillStyle;
        this.context.fill();
      }
      return this.context.closePath();
    };

    Graph.prototype.renderCurve = function(curve, result, xPos, yPos, calculatorIdx) {
      var fillStyle, i, k, len, ref, setOfPoints;
      this.context.beginPath();
      this.context.setLineDash(curve.lineDash(calculatorIdx));
      if (result.multiplePaths) {
        ref = result.points;
        for (i = k = 0, len = ref.length; k < len; i = ++k) {
          setOfPoints = ref[i];
          this.drawCurve(setOfPoints, result.tension, false, xPos, yPos);
        }
      } else {
        this.drawCurve(result.points, result.tension, false, xPos, yPos);
      }
      if (curve.strokeStyle()) {
        this.context.lineWidth = curve.get('lineWidth');
        this.context.strokeStyle = curve.strokeStyle();
        this.context.stroke();
      }
      if (fillStyle = curve.get('fillStyle')) {
        this.context.fillStyle = fillStyle;
        this.context.fill();
      }
      return this.context.closePath();
    };

    Graph.prototype.renderGuide = function(guide) {
      var valueAtPoint, xPos, yPos;
      valueAtPoint = this.model.get(guide.attribute);
      this.context.beginPath();
      this.context.setLineDash([]);
      this.context.strokeStyle = '#999999';
      this.context.lineWidth = 2;
      this.context.moveTo(this.width, 0);
      this.context.lineTo(0, 0);
      if (guide.orientation === 'vertical') {
        this.context.moveTo(0, this.height);
        this.context.lineTo(this.width, this.height);
        this.context.lineTo(this.width, this.height - 10);
        xPos = this.xOrigin + valueAtPoint * this.width / this.xRange;
        this.context.moveTo(xPos, 0);
        this.context.lineTo(xPos, this.height);
      } else {
        this.context.lineTo(0, this.height);
        yPos = -valueAtPoint * this.height / this.yRange + this.yOrigin;
        this.context.moveTo(0, yPos);
        this.context.lineTo(this.width, yPos);
      }
      this.context.stroke();
      this.context.closePath();
      this.context.beginPath();
      this.context.fillStyle = "#f2f2f2";
      this.context.lineWidth = 1;
      if (guide.orientation === 'vertical') {
        this.context.moveTo(xPos + 8, 0);
        this.context.lineTo(xPos, 8);
        this.context.lineTo(xPos - 8, 0);
        this.context.moveTo(xPos - 8, this.height);
        this.context.lineTo(xPos, this.height - 8);
        this.context.lineTo(xPos + 8, this.height);
      } else {
        this.context.moveTo(0, yPos - 8);
        this.context.lineTo(8, yPos);
        this.context.lineTo(0, yPos + 8);
        this.context.moveTo(this.width, yPos - 8);
        this.context.lineTo(this.width - 8, yPos);
        this.context.lineTo(this.width, yPos + 8);
      }
      this.context.stroke();
      this.context.fill();
      return this.context.closePath();
    };

    Graph.prototype.render = function() {
      var guide, k, l, len, len1, len2, m, ref, x, xAxisY, y, yAxisX;
      this.width = this.params.get('width');
      this.height = this.params.get('height');
      this.xOrigin = this.params.get('xOrigin');
      this.yOrigin = this.params.get('yOrigin');
      this.yOffset = this.params.get('yOffset');
      this.xRange = this.params.get('xRange');
      this.yRange = this.params.get('yRange');
      this.xAxisScale = this.params.get('xAxisScale');
      this.yAxisScale = this.params.get('yAxisScale');
      this.xMin = -this.xOrigin * this.xRange / this.width;
      this.xMax = (this.width - this.xOrigin) * this.xRange / this.width;
      this.yMin = -(this.height - this.yOrigin) * this.yRange / this.height;
      this.yMax = this.yOrigin * this.yRange / this.height;
      if (this.width === null && this.height === null) {
        return this;
      }
      this.context || (this.context = this.el.getContext('2d'));
      this.context.clearRect(0, 0, this.width, this.height);
      this.beforeRender();
      this.renderGrid();
      xAxisY = this.params.get('xAxisY');
      if ($.isArray(xAxisY)) {
        for (k = 0, len = xAxisY.length; k < len; k++) {
          y = xAxisY[k];
          this.renderXAxis(y);
        }
      } else {
        this.renderXAxis(xAxisY);
      }
      yAxisX = this.params.get('yAxisX');
      if ($.isArray(yAxisX)) {
        for (l = 0, len1 = yAxisX.length; l < len1; l++) {
          x = yAxisX[l];
          this.renderYAxis(x);
        }
      } else {
        this.renderYAxis(yAxisX);
      }
      this.renderCurves();
      ref = this.params.get('guides');
      for (m = 0, len2 = ref.length; m < len2; m++) {
        guide = ref[m];
        this.renderGuide(guide);
      }
      return this;
    };

    return Graph;

  })(ELA.Views.Canvas);

}).call(this);
