(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  if (ELA.Views == null) {
    ELA.Views = {};
  }

  ELA.Views.GraphView = (function(superClass) {
    extend(GraphView, superClass);

    function GraphView() {
      this.render = bind(this.render, this);
      return GraphView.__super__.constructor.apply(this, arguments);
    }

    GraphView.prototype.className = ELA.Views.ViewportView.prototype.className + " graph-view";

    GraphView.prototype.initialize = function(options) {
      var axis, base, name, props, ref, ref1, ref10, ref11, ref12, ref13, ref14, ref2, ref3, ref4, ref5, ref6, ref7, ref8, ref9;
      if (options == null) {
        options = {};
      }
      if (options.name == null) {
        throw 'ELA.Views.GraphView: option `name` is required';
      }
      if (options.legend !== false) {
        if (_.isObject(options.legend) && (options.legend.view != null)) {
          this.LegendView = options.legend.view.toFunction();
          this.legendValueAttribute = options.legend.valueAttribute;
        } else {
          this.LegendView = ELA.Views.Legend;
        }
        if (this.legendValueAttribute == null) {
          this.legendValueAttribute = (ref = options.graph) != null ? (ref1 = ref.axes) != null ? (ref2 = ref1.x) != null ? ref2.attribute : void 0 : void 0 : void 0;
        }
      }
      if (((ref3 = options.graphOverlay) != null ? ref3.view : void 0) != null) {
        this.GraphOverlayView = options.graphOverlay.view.toFunction();
      }
      if (((ref4 = options.graph) != null ? ref4.axes : void 0) != null) {
        if (options.graph.axes.x != null) {
          this.xAxisY = options.graph.axes.x.y;
          this.xAxisScale = options.graph.axes.x.scale;
        }
        if (options.graph.axes.y != null) {
          this.yAxisX = options.graph.axes.y.x;
          this.yAxisScale = options.graph.axes.y.scale;
        }
      }
      ref6 = (ref5 = options.graph) != null ? ref5.axes : void 0;
      for (axis in ref6) {
        props = ref6[axis];
        if (props.handler) {
          switch (axis) {
            case 'x':
              this.bottomAxisHandler = {
                attribute: props.attribute
              };
              break;
            case 'y':
              this.leftAxisHandler = {
                attribute: props.attribute
              };
          }
        }
      }
      this.guides = [];
      if (this.bottomAxisHandler != null) {
        this.guides.push({
          orientation: 'vertical',
          attribute: this.bottomAxisHandler.attribute
        });
      }
      if (this.leftAxisHandler != null) {
        this.guides.push({
          orientation: 'horizontal',
          attribute: this.leftAxisHandler.attribute
        });
      }
      if (((ref7 = options.graph) != null ? ref7.curves : void 0) != null) {
        this.curves = options.graph.curves.slice();
        this.axisLabelingForCurve = this.model.curves.find((function(_this) {
          return function(curve) {
            return curve.get('function') === _this.curves[0];
          };
        })(this));
      } else {
        this.axisLabelingForCurve = this.model.curves.first();
      }
      if ((ref8 = options.graph) != null ? ref8.view : void 0) {
        this.GraphView = options.graph.view.toFunction();
      } else {
        this.GraphView = ELA.Views.Graph;
      }
      this.displayParams = (base = this.model.displayParams)[name = options.name] != null ? base[name] : base[name] = new this.GraphView.Params({
        guides: this.guides,
        curves: this.curves,
        axisLabelingForCurve: this.axisLabelingForCurve,
        xAxisY: this.xAxisY,
        xAxisScale: this.xAxisScale,
        yAxisX: this.yAxisX,
        yAxisScale: this.yAxisScale,
        app: this.model,
        xOrigin: (ref9 = options.graph) != null ? ref9.xOrigin : void 0,
        yOrigin: (ref10 = options.graph) != null ? ref10.yOrigin : void 0,
        xScale: (ref11 = options.graph) != null ? ref11.xScale : void 0,
        yScale: (ref12 = options.graph) != null ? ref12.yScale : void 0,
        xOriginRatio: (ref13 = options.graph) != null ? ref13.xOriginRatio : void 0,
        yOriginRatio: (ref14 = options.graph) != null ? ref14.yOriginRatio : void 0
      });
      return this.subviews = {};
    };

    GraphView.prototype.render = function() {
      var $a, $graph, $horizontalWrapper, base, base1, base2, base3, view;
      GraphView.__super__.render.apply(this, arguments);
      if (this.LegendView != null) {
        view = (base = this.subviews).legend != null ? base.legend : base.legend = new this.LegendView({
          model: this.model,
          parentView: this,
          localePrefix: this.localePrefix,
          valueAttribute: this.legendValueAttribute,
          curves: this.curves,
          displayParams: this.displayParams
        });
        this.$el.append(view.render().el);
      }
      if (this.GraphOverlayView != null) {
        view = (base1 = this.subviews).graphOverlay != null ? base1.graphOverlay : base1.graphOverlay = new this.GraphOverlayView({
          model: this.model,
          parentView: this,
          localePrefix: this.localePrefix
        });
        this.$el.append(view.render().el);
      }
      $horizontalWrapper = $('<div>', {
        "class": 'horizontal-wrapper'
      });
      if (this.leftAxisHandler != null) {
        view = (base2 = this.subviews).leftAxisHandler != null ? base2.leftAxisHandler : base2.leftAxisHandler = new ELA.Views.AxisHandler({
          model: this.model,
          displayParams: this.displayParams,
          attribute: this.leftAxisHandler.attribute,
          position: 'left',
          parentView: this,
          localePrefix: this.localePrefix
        });
        $horizontalWrapper.append(view.render().el);
      }
      $graph = $('<div>', {
        "class": 'graph'
      });
      $a = $('<a>', {
        id: 'upstream',
        href: ELA.settings.upstream.url,
        target: '_blank'
      });
      $a.html($('<img>', {
        src: 'images/logo.png'
      }));
      $horizontalWrapper.append($graph.html($a));
      this.$el.append($horizontalWrapper);
      if (this.bottomAxisHandler != null) {
        view = (base3 = this.subviews).bottomAxisHandler != null ? base3.bottomAxisHandler : base3.bottomAxisHandler = new ELA.Views.AxisHandler({
          model: this.model,
          displayParams: this.displayParams,
          attribute: this.bottomAxisHandler.attribute,
          position: 'bottom',
          parentView: this,
          localePrefix: this.localePrefix
        });
        this.$el.append(view.render().el);
      }
      delay((function(_this) {
        return function() {
          var ref;
          if ((ref = _this.subviews.graph) != null) {
            ref.remove();
          }
          _this.displayParams.set({
            width: $graph[0].clientWidth,
            height: $graph[0].clientHeight
          });
          view = _this.subviews.graph = new _this.GraphView({
            model: _this.model,
            parentView: _this,
            params: _this.displayParams,
            localePrefix: _this.localePrefix
          });
          if (_this.leftAxisHandler != null) {
            view.$el.on('tap', function(e) {
              var ref1;
              if (!((ref1 = _this.subviews.bottomAxisHandler) != null ? ref1.hasRecentlyOpenInput() : void 0)) {
                return _this.subviews.leftAxisHandler.handleTap(e);
              }
            });
          }
          if (_this.bottomAxisHandler != null) {
            view.$el.on('tap', function(e) {
              var ref1;
              if (!((ref1 = _this.subviews.leftAxisHandler) != null ? ref1.hasRecentlyOpenInput() : void 0)) {
                return _this.subviews.bottomAxisHandler.handleTap(e);
              }
            });
          }
          return $graph.append(view.render().el);
        };
      })(this));
      return this;
    };

    return GraphView;

  })(ELA.Views.ViewportView);

}).call(this);
