(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  if (ELA.Views == null) {
    ELA.Views = {};
  }

  ELA.Views.Help = (function(superClass) {
    extend(Help, superClass);

    function Help() {
      this.render = bind(this.render, this);
      this.renderArticle = bind(this.renderArticle, this);
      this.renderMath = bind(this.renderMath, this);
      this.scrollTo = bind(this.scrollTo, this);
      this.scrollToToc = bind(this.scrollToToc, this);
      this.remove = bind(this.remove, this);
      this.scrollToAnchor = bind(this.scrollToAnchor, this);
      this.goBack = bind(this.goBack, this);
      return Help.__super__.constructor.apply(this, arguments);
    }

    Help.prototype.id = 'help';

    Help.prototype.tagName = 'section';

    Help.prototype.template = JST['general/help'];

    Help.prototype.typeset = false;

    Help.prototype.scrollTarget = null;

    Help.prototype.events = {
      'tap header .back.icon': 'goBack',
      'tap header h2': 'scrollToToc',
      'click a': 'scrollToAnchor'
    };

    Help.prototype.hammerjs = true;

    Help.prototype.goBack = function(e) {
      if (this.model.path != null) {
        return ELA.router.navigate("app/" + this.model.path, {
          trigger: true
        });
      } else {
        return ELA.router.navigate('/', {
          trigger: true
        });
      }
    };

    Help.prototype.scrollToAnchor = function(e) {
      var $target, href;
      href = $(e.currentTarget).attr('href');
      if (href.match(/^#(.*)$/)) {
        $target = this.$article.find(href);
        if ($target.length > 0) {
          e.preventDefault();
          this.scrollTo(href);
          return this.scrollTarget = href;
        }
      }
    };

    Help.prototype.remove = function() {
      return this.$el.afterTransitionForRemovingClass('active', (function(_this) {
        return function() {
          return Help.__super__.remove.apply(_this, arguments);
        };
      })(this));
    };

    Help.prototype.scrollToToc = function(e) {
      this.scrollTo('.toc');
      return e.stopPropagation();
    };

    Help.prototype.setActive = function(val) {
      return this.$el.toggleClass('active', val);
    };

    Help.prototype.activate = function() {
      return this.$el.toggleClass('active', true);
    };

    Help.prototype.deactivate = function() {
      return this.$el.toggleClass('active', false);
    };

    Help.prototype.scrollTo = function(target) {
      return this.$article.scrollTo(target, 600, {
        axis: 'y',
        offset: -5
      });
    };

    Help.prototype.renderMath = function() {
      var performTypeset, typesetCallback;
      typesetCallback = (function(_this) {
        return function() {
          _this.$el.find('.toast').addClass('hidden');
          if (_this.scrollTarget != null) {
            return _this.scrollTo(_this.scrollTarget);
          }
        };
      })(this);
      performTypeset = (function(_this) {
        return function() {
          if (window.MathJax.isReady) {
            return MathJax.Hub.Queue(["Typeset", MathJax.Hub, _this.$article[0], typesetCallback]);
          } else {
            return setTimeout(performTypeset, 100);
          }
        };
      })(this);
      return performTypeset();
    };

    Help.prototype.renderArticle = function(text) {
      if (!ELA.converter) {
        ELA.converter = new Markdown.Converter();
        Markdown.Extra.init(ELA.converter);
      }
      this.$article.html(ELA.converter.makeHtml(text));
      Markdown.Toc.apply(this.$article);
      return this.renderMath();
    };

    Help.prototype.render = function() {
      var base, fileName;
      this.$el.html(this.template());
      this.$article = this.$el.find('article');
      if (!this.loadingOrLoadedSuccessfully) {
        this.loadingOrLoadedSuccessfully = true;
        fileName = (typeof (base = this.model).helpTextName === "function" ? base.helpTextName() : void 0) || 'about';
        $.ajax("help/" + fileName + ".md", {
          success: this.renderArticle,
          error: (function(_this) {
            return function() {
              _this.loadingOrLoadedSuccessfully = false;
              _this.$el.find('.toast').addClass('hidden');
              return _this.$article.html(t('help.messages.missing_help_text'));
            };
          })(this)
        });
      }
      return this;
    };

    return Help;

  })(Backbone.View);

}).call(this);
