(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  if (ELA.Views == null) {
    ELA.Views = {};
  }

  ELA.Views.InterpolatedGraph = (function(superClass) {
    extend(InterpolatedGraph, superClass);

    function InterpolatedGraph() {
      this.render = bind(this.render, this);
      this.maxRangeY = bind(this.maxRangeY, this);
      this.maxRangeX = bind(this.maxRangeX, this);
      return InterpolatedGraph.__super__.constructor.apply(this, arguments);
    }

    InterpolatedGraph.prototype.initialize = function(options) {
      var guide, k, len, ref;
      if (options == null) {
        options = {};
      }
      InterpolatedGraph.__super__.initialize.apply(this, arguments);
      this.bindCalculatorEvents();
      this.model.curves.on('change:selected', (function(_this) {
        return function() {
          _this.bindCalculatorEvents();
          return _this.requestRepaint();
        };
      })(this));
      this.model.on('change:calculators', (function(_this) {
        return function() {
          _this.calculateRanges();
          _this.bindCalculatorEvents();
          return _this.requestRepaint();
        };
      })(this));
      ref = this.params.get('guides');
      for (k = 0, len = ref.length; k < len; k++) {
        guide = ref[k];
        this.listenTo(this.model, "change:" + guide.attribute, this.requestRepaint);
      }
      return this.listenTo(this.params, 'change:axisLabelingForCurve', this.requestRepaint);
    };

    InterpolatedGraph.prototype.bindCalculatorEvents = function() {
      var calc, curve, filteredCurves, k, l, len, len1, oldestCurve, ref, results;
      this.stopListening(this.model.previous('calculators'));
      ref = this.model.get('calculators');
      results = [];
      for (k = 0, len = ref.length; k < len; k++) {
        calc = ref[k];
        filteredCurves = this._filteredCurves();
        for (l = 0, len1 = filteredCurves.length; l < len1; l++) {
          curve = filteredCurves[l];
          this.listenTo(calc, "change:" + (curve.get('function')), this.requestRepaint);
        }
        this.listenTo(calc, 'change:maxX change:xRange', this.calculateRangeX);
        this.listenTo(calc, 'change:maxY change:yRange', this.calculateRangeY);
        oldestCurve = filteredCurves[0];
        if (oldestCurve != null) {
          results.push(this.listenTo(calc, "change:" + (oldestCurve.get('function')), this.calculateRanges));
        } else {
          results.push(void 0);
        }
      }
      return results;
    };

    InterpolatedGraph.prototype.maxRangeX = function(func, xScale) {
      var calc, ranges, ref;
      if (xScale == null) {
        xScale = this.params.get('xScale');
      }
      if (func == null) {
        func = (ref = this.params.get('axisLabelingForCurve')) != null ? ref.get('function') : void 0;
      }
      ranges = (function() {
        var k, len, ref1, results;
        ref1 = this.model.get('calculators');
        results = [];
        for (k = 0, len = ref1.length; k < len; k++) {
          calc = ref1[k];
          results.push(calc.xRange(func));
        }
        return results;
      }).call(this);
      return Math.max.apply(Math, _.compact(ranges)) * xScale;
    };

    InterpolatedGraph.prototype.maxRangeY = function(func, yScale) {
      var calc, ranges, ref;
      if (yScale == null) {
        yScale = this.params.get('yScale');
      }
      if (func == null) {
        func = (ref = this.params.get('axisLabelingForCurve')) != null ? ref.get('function') : void 0;
      }
      ranges = (function() {
        var k, len, ref1, results;
        ref1 = this.model.get('calculators');
        results = [];
        for (k = 0, len = ref1.length; k < len; k++) {
          calc = ref1[k];
          results.push(calc.yRange(func));
        }
        return results;
      }).call(this);
      return Math.max.apply(Math, _.compact(ranges)) * yScale;
    };

    InterpolatedGraph.prototype.xAxisValueLabel = function(val, stepsize) {
      return this.axisLabel(val, stepsize);
    };

    InterpolatedGraph.prototype.yAxisValueLabel = function(val, stepsize) {
      var curve, curvePresenter;
      curve = this.params.get('axisLabelingForCurve');
      if (curve != null) {
        curvePresenter = this.Present(curve);
        val = curvePresenter.unitValue(val);
        stepsize = curvePresenter.unitValue(stepsize);
      }
      return this.axisLabel(val - this.params.get('yOffset'), stepsize);
    };

    InterpolatedGraph.prototype.genericAxisLabel = function(axis) {
      var axisLabel, axisLabelLocale, curve;
      axisLabel = this.params.get(axis + "AxisLabel");
      if ((axisLabel != null) && _.isFunction(axisLabel)) {
        return axisLabel.call(this);
      }
      if (axisLabel != null) {
        return axisLabel;
      }
      axisLabelLocale = this.params.get(axis + "AxisLabelLocale");
      if (axisLabelLocale != null) {
        return this.loadLocale(axisLabelLocale);
      }
      curve = this.params.get('axisLabelingForCurve');
      if (curve != null) {
        if (axis === 'x') {
          return this.Present(curve).fullXAxisLabel();
        }
        if (axis === 'y') {
          return this.Present(curve).fullYAxisLabel();
        }
      }
      return this.loadLocale("graph." + axis + "AxisLabel", {
        defaultValue: ''
      });
    };

    InterpolatedGraph.prototype.xAxisLabel = function() {
      return this.genericAxisLabel('x');
    };

    InterpolatedGraph.prototype.yAxisLabel = function() {
      return this.genericAxisLabel('y');
    };

    InterpolatedGraph.prototype.getControlPoints = function(x0, y0, x1, y1, x2, y2, tension) {
      var d01, d12, fa, fb, p1x, p1y, p2x, p2y;
      d01 = Math.sqrt(Math.pow(x1 - x0, 2) + Math.pow(y1 - y0, 2));
      d12 = Math.sqrt(Math.pow(x2 - x1, 2) + Math.pow(y2 - y1, 2));
      fa = tension * d01 / (d01 + d12);
      fb = tension - fa;
      p1x = x1 + fa * (x0 - x2);
      p1y = y1 + fa * (y0 - y2);
      p2x = x1 - fb * (x0 - x2);
      p2y = y1 - fb * (y0 - y2);
      return [p1x, p1y, p2x, p2y];
    };

    InterpolatedGraph.prototype.drawSpline = function(points, t, closed, xPos, yPos) {
      var cp, i, k, l, m, n, o, ref, ref1, ref2, ref3, results;
      if (xPos == null) {
        xPos = (function(x) {
          return x;
        });
      }
      if (yPos == null) {
        yPos = (function(y) {
          return y;
        });
      }
      if (points.length <= 4) {
        this.context.moveTo(xPos(points[0]), yPos(points[1]));
        return this.context.lineTo(xPos(points[2]), yPos(points[3]));
      } else {
        cp = [];
        n = points.length;
        points = _.clone(points);
        if (closed) {
          points.push(points[0], points[1], points[2], points[3]);
          points.unshift(points[n - 1]);
          points.unshift(points[n - 1]);
          for (i = k = 0, ref = n; k <= ref; i = k += 2) {
            cp = cp.concat(this.getControlPoints(points[i], points[i + 1], points[i + 2], points[i + 3], points[i + 4], points[i + 5], t));
          }
          cp = cp.concat(cp[0], cp[1]);
          results = [];
          for (i = l = 0, ref1 = n; l <= ref1; i = l += 2) {
            this.context.moveTo(xPos(points[i]), yPos(points[i + 1]));
            results.push(this.context.bezierCurveTo(xPos(cp[2 * i - 2]), yPos(cp[2 * i - 1]), xPos(cp[2 * i]), yPos(cp[2 * i + 1]), xPos(points[i + 2]), yPos(points[i + 3])));
          }
          return results;
        } else {
          for (i = m = 0, ref2 = n - 4; m <= ref2; i = m += 2) {
            cp = cp.concat(this.getControlPoints(points[i], points[i + 1], points[i + 2], points[i + 3], points[i + 4], points[i + 5], t));
          }
          this.context.moveTo(xPos(points[0]), yPos(points[1]));
          this.context.quadraticCurveTo(xPos(cp[0]), yPos(cp[1]), xPos(points[2]), yPos(points[3]));
          for (i = o = 2, ref3 = n - 3; o <= ref3; i = o += 2) {
            this.context.moveTo(xPos(points[i]), yPos(points[i + 1]));
            this.context.bezierCurveTo(xPos(cp[2 * i - 2]), yPos(cp[2 * i - 1]), xPos(cp[2 * i]), yPos(cp[2 * i + 1]), xPos(points[i + 2]), yPos(points[i + 3]));
          }
          this.context.moveTo(xPos(points[n - 2]), yPos(points[n - 1]));
          return this.context.quadraticCurveTo(xPos(cp[2 * n - 10]), yPos(cp[2 * n - 9]), xPos(points[n - 4]), yPos(points[n - 3]));
        }
      }
    };

    InterpolatedGraph.prototype.drawCurve = function(points, tension, closed, xPos, yPos) {
      var beginning, i, k, ref, results;
      if (tension == null) {
        tension = 0;
      }
      if (closed == null) {
        closed = false;
      }
      if (xPos == null) {
        xPos = (function(x) {
          return x;
        });
      }
      if (yPos == null) {
        yPos = (function(y) {
          return y;
        });
      }
      if (tension === 0) {
        beginning = true;
        if (points.length >= 2) {
          this.context.moveTo(xPos(points[0]), yPos(points[1]));
          results = [];
          for (i = k = 2, ref = points.length; k < ref; i = k += 2) {
            results.push(this.context.lineTo(xPos(points[i]), yPos(points[i + 1])));
          }
          return results;
        }
      } else {
        return this.drawSpline(points, tension, closed, xPos, yPos);
      }
    };

    InterpolatedGraph.prototype._filteredCurves = function() {
      var curves, graphCurves;
      curves = this.model.curves.history;
      if (graphCurves = this.params.get('curves')) {
        return _.filter(curves, function(curve) {
          return graphCurves.indexOf(curve.get('function')) >= 0;
        });
      } else {
        return curves;
      }
    };

    InterpolatedGraph.prototype._sortedCurves = function() {
      return _.sortBy(this._filteredCurves(), function(curve) {
        return curve.get('zIndex');
      });
    };

    InterpolatedGraph.prototype.beforeRenderCurves = function(resultCurves) {};

    InterpolatedGraph.prototype.afterRenderCurves = function(resultCurves) {};

    InterpolatedGraph.prototype.renderCurves = function() {
      var calc, curve, func, i, j, k, l, len, len1, len2, m, name, ref, ref1, ref2, result, resultCurve, resultCurves, subcurve, xPos, xRange, yPos, yRange;
      resultCurves = [];
      ref = this.model.get('calculators');
      for (i = k = 0, len = ref.length; k < len; i = ++k) {
        calc = ref[i];
        ref1 = this._sortedCurves();
        for (j = l = 0, len1 = ref1.length; l < len1; j = ++l) {
          curve = ref1[j];
          func = curve.get('function');
          resultCurves.push({
            curve: curve,
            result: calc[func](this.xMin, this.xMax),
            yRange: this.maxRangeY(func),
            xRange: this.maxRangeX(func),
            calculatorIdx: i
          });
        }
      }
      this.beforeRenderCurves(resultCurves);
      for (m = 0, len2 = resultCurves.length; m < len2; m++) {
        resultCurve = resultCurves[m];
        curve = resultCurve.curve;
        result = resultCurve.result;
        yRange = resultCurve.yRange;
        xRange = resultCurve.xRange;
        if (_.isObject(result)) {
          xPos = (function(_this) {
            return function(x) {
              return _this.xOrigin + x * _this.width / xRange;
            };
          })(this);
          yPos = (function(_this) {
            return function(y) {
              return _this.yOrigin - (y + _this.yOffset) * _this.height / yRange;
            };
          })(this);
          if (curve.hasSubcurves()) {
            ref2 = curve.subcurves();
            for (name in ref2) {
              subcurve = ref2[name];
              if (result[name].points != null) {
                this.renderCurve(subcurve, result[name], xPos, yPos, resultCurve.calculatorIdx);
              } else if (result[name].radius != null) {
                this.renderCircle(subcurve, result[name], xPos, yPos, resultCurve.calculatorIdx);
              }
            }
          } else {
            if (result.points != null) {
              this.renderCurve(curve, result, xPos, yPos, resultCurve.calculatorIdx);
            } else if (result.radius != null) {
              this.renderCircle(curve, result, xPos, yPos, resultCurve.calculatorIdx);
            }
          }
        }
      }
      return this.afterRenderCurves(resultCurves);
    };

    InterpolatedGraph.prototype.renderCircle = function(curve, result, xPos, yPos, calculatorIdx) {
      var aX, aY, centerX, centerY, eX, eY, fillStyle, func, hB, height, mX, mY, vB, width, yRange;
      func = curve.get('function');
      yRange = this.maxRangeY(func);
      this.context.beginPath();
      width = Math.abs(xPos(2 * result.radius) - this.xOrigin);
      height = Math.abs(yPos(2 * result.radius) - this.yOrigin);
      centerX = xPos(result.center[0]);
      centerY = yPos(result.center[1]);
      aX = centerX - width / 2;
      aY = centerY - height / 2;
      hB = (width / 2) * .5522848;
      vB = (height / 2) * .5522848;
      eX = aX + width;
      eY = aY + height;
      mX = aX + width / 2;
      mY = aY + height / 2;
      this.context.moveTo(aX, mY);
      this.context.bezierCurveTo(aX, mY - vB, mX - hB, aY, mX, aY);
      this.context.bezierCurveTo(mX + hB, aY, eX, mY - vB, eX, mY);
      this.context.bezierCurveTo(eX, mY + vB, mX + hB, eY, mX, eY);
      this.context.bezierCurveTo(mX - hB, eY, aX, mY + vB, aX, mY);
      this.context.setLineDash(curve.lineDash(calculatorIdx));
      if (curve.strokeStyle()) {
        this.context.lineWidth = curve.get('lineWidth');
        this.context.strokeStyle = curve.strokeStyle();
        this.context.stroke();
      }
      if (fillStyle = curve.get('fillStyle')) {
        this.context.fillStyle = fillStyle;
        this.context.fill();
      }
      return this.context.closePath();
    };

    InterpolatedGraph.prototype.renderCurve = function(curve, result, xPos, yPos, calculatorIdx) {
      var fillStyle, i, k, len, ref, setOfPoints;
      this.context.beginPath();
      this.context.setLineDash(curve.lineDash(calculatorIdx));
      if (result.multiplePaths) {
        ref = result.points;
        for (i = k = 0, len = ref.length; k < len; i = ++k) {
          setOfPoints = ref[i];
          this.drawCurve(setOfPoints, result.tension, false, xPos, yPos);
        }
      } else {
        this.drawCurve(result.points, result.tension, false, xPos, yPos);
      }
      if (curve.strokeStyle()) {
        this.context.lineWidth = curve.get('lineWidth');
        this.context.strokeStyle = curve.strokeStyle();
        this.context.stroke();
      }
      if (fillStyle = curve.get('fillStyle')) {
        this.context.fillStyle = fillStyle;
        this.context.fill();
      }
      return this.context.closePath();
    };

    InterpolatedGraph.prototype.renderGuide = function(guide) {
      var valueAtPoint, xPos, yPos;
      valueAtPoint = this.model.get(guide.attribute);
      this.context.beginPath();
      this.context.setLineDash([]);
      this.context.strokeStyle = '#999999';
      this.context.lineWidth = 2;
      this.context.moveTo(this.width, 0);
      this.context.lineTo(0, 0);
      if (guide.orientation === 'vertical') {
        this.context.moveTo(0, this.height);
        this.context.lineTo(this.width, this.height);
        this.context.lineTo(this.width, this.height - 10);
        xPos = this.xOrigin + valueAtPoint * this.width / this.xRange;
        this.context.moveTo(xPos, 0);
        this.context.lineTo(xPos, this.height);
      } else {
        this.context.lineTo(0, this.height);
        yPos = -valueAtPoint * this.height / this.yRange + this.yOrigin;
        this.context.moveTo(0, yPos);
        this.context.lineTo(this.width, yPos);
      }
      this.context.stroke();
      this.context.closePath();
      this.context.beginPath();
      this.context.fillStyle = "#f2f2f2";
      this.context.lineWidth = 1;
      if (guide.orientation === 'vertical') {
        this.context.moveTo(xPos + 8, 0);
        this.context.lineTo(xPos, 8);
        this.context.lineTo(xPos - 8, 0);
        this.context.moveTo(xPos - 8, this.height);
        this.context.lineTo(xPos, this.height - 8);
        this.context.lineTo(xPos + 8, this.height);
      } else {
        this.context.moveTo(0, yPos - 8);
        this.context.lineTo(8, yPos);
        this.context.lineTo(0, yPos + 8);
        this.context.moveTo(this.width, yPos - 8);
        this.context.lineTo(this.width - 8, yPos);
        this.context.lineTo(this.width, yPos + 8);
      }
      this.context.stroke();
      this.context.fill();
      return this.context.closePath();
    };

    InterpolatedGraph.prototype.render = function() {
      var guide, k, len, ref;
      InterpolatedGraph.__super__.render.apply(this, arguments);
      ref = this.params.get('guides');
      for (k = 0, len = ref.length; k < len; k++) {
        guide = ref[k];
        this.renderGuide(guide);
      }
      return this;
    };

    return InterpolatedGraph;

  })(ELA.Views.BaseGraph);

}).call(this);
