(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  Backbone.Poised || (Backbone.Poised = {});

  Backbone.Poised.Form = (function(superClass) {
    extend(Form, superClass);

    function Form() {
      this.render = bind(this.render, this);
      this.appendItem = bind(this.appendItem, this);
      this.appendGroup = bind(this.appendGroup, this);
      this.cancel = bind(this.cancel, this);
      this.submit = bind(this.submit, this);
      this.validateSubmit = bind(this.validateSubmit, this);
      return Form.__super__.constructor.apply(this, arguments);
    }

    Form.prototype.tagName = 'form';

    Form.prototype.className = 'poised';

    Form.prototype.events = {
      'submit': 'preventOriginalSubmit'
    };

    Form.prototype.preventOriginalSubmit = function(e) {
      return e.preventDefault();
    };

    Form.prototype.extractGroupOptions = function(options) {
      if (options == null) {
        options = {};
      }
      return this.options.group = _.chain(options).pick('by', 'collapsible', 'collapsed').defaults({
        collapsible: false,
        collapsed: []
      }).value();
    };

    Form.prototype.extractOptions = function(options) {
      if (options == null) {
        options = {};
      }
      this.options = _.chain(options).pick('cancelable', 'resetable', 'title', 'creationTitle', 'editingTitle', 'liveForm', 'validate').defaults({
        cancelable: false,
        resetable: false,
        liveForm: false,
        validate: this.model.__proto__.validation != null
      }).value();
      return this.extractGroupOptions(options.group);
    };

    Form.prototype.extractFields = function(options) {
      if (options == null) {
        options = {};
      }
      if (options.fields) {
        return this.fields = options.fields;
      } else if (this.model.formFields != null) {
        return this.fields = this.model.formFields;
      } else {
        return this.fields = [];
      }
    };

    Form.prototype.initialize = function(options) {
      if (options == null) {
        options = {};
      }
      if (options.model == null) {
        throw new Error('Missing `model` option');
      }
      this.extractOptions(options);
      return this.extractFields(options);
    };

    Form.prototype.validateSubmit = function() {
      var errors;
      errors = this.model.validate();
      if (_.isEmpty(errors)) {
        return this.trigger('validSubmit', this.model);
      } else {
        this.trigger('invalidSubmit', this.model);
        this.$('.invalid input').first().blur();
        return this.$('.invalid input').first().focus();
      }
    };

    Form.prototype.submit = function() {
      this.trigger('submit', this.model);
      if (this.options.validate) {
        return this.validateSubmit();
      }
    };

    Form.prototype.cancel = function() {
      return this.trigger('cancel', this.model);
    };

    Form.prototype.appendGroup = function(group, fields) {
      var $grp, field, i, len;
      $grp = $('<div>', {
        "class": 'poised control-group'
      });
      for (i = 0, len = fields.length; i < len; i++) {
        field = fields[i];
        this.appendItem(field, $grp);
      }
      return this.$el.append($grp);
    };

    Form.prototype.appendItem = function(field, $el) {
      var f;
      if ($el == null) {
        $el = this.$el;
      }
      f = _.clone(field);
      if (f.anchor != null) {
        f.type = 'anchor';
      }
      if (f.range != null) {
        f.type = 'range';
      }
      if (f.options != null) {
        f.type = 'select';
      }
      f.type || (f.type = 'text');
      f.locale = this.locale;
      f.localePrefix = this.localePrefix;
      this.subviews[f.attribute] = this.controlView(f);
      return $el.append(this.subviews[f.attribute].render().el);
    };

    Form.prototype.controlView = function(field) {
      field = _.defaults(field, {
        model: this.model,
        parentView: this
      });
      field.validate = this.model.__proto__.validation && _.has(this.model.__proto__.validation, field.attribute);
      switch (field.type) {
        case 'text':
          return new Backbone.Poised.StringControl(field);
        case 'range':
          return new Backbone.Poised.RangeControl(field);
        case 'number':
          return new Backbone.Poised.NumberControl(field);
        case 'select':
          return new Backbone.Poised.SelectControl(field);
        case 'value':
          return new Backbone.Poised.ValueControl(field);
        case 'anchor':
          return new Backbone.Poised.Anchor(field);
      }
    };

    Form.prototype.bindValidation = function() {
      if (this.options.validate) {
        return Backbone.Validation.bind(this);
      }
    };

    Form.prototype.title = function() {
      if (this.model.isNew()) {
        return this.loadLocale('formTitles.create', 'formTitle', {
          defaultValue: this.options.creationTitle || this.options.title,
          returnNull: true
        });
      } else {
        return this.loadLocale('formTitles.edit', 'formTitle', {
          defaultValue: this.options.editingTitle || this.options.title,
          returnNull: true
        });
      }
    };

    Form.prototype.render = function() {
      var field, group, groups, i, j, len, len1, ref, title;
      this.$el.html();
      title = this.title();
      if (title) {
        this.$el.append($('<h2 />', {
          text: title
        }));
      }
      if (this.options.group.by) {
        groups = _.chain(this.fields).pluck(this.options.group.by).compact().unique().value();
        for (i = 0, len = groups.length; i < len; i++) {
          group = groups[i];
          this.appendItem({
            type: 'anchor',
            anchor: group.toCamel(),
            collapsible: $.ematch(group, this.options.group.collapsible),
            collapsed: $.ematch(group, this.options.group.collapsed)
          });
          this.appendGroup(group, _.filter(this.fields, (function(_this) {
            return function(f) {
              return f[_this.options.group.by] === group;
            };
          })(this)));
        }
      } else {
        ref = this.fields;
        for (j = 0, len1 = ref.length; j < len1; j++) {
          field = ref[j];
          this.appendItem(field);
        }
      }
      if (!this.options.liveForm) {
        this.subviews.buttons = new Backbone.Poised.SubmitControl({
          parentView: this,
          cancelable: this.options.cancelable,
          resetable: this.options.resetable
        });
        this.listenTo(this.subviews.buttons, 'submit', this.submit);
        this.listenTo(this.subviews.buttons, 'cancel', this.cancel);
        this.$el.append(this.subviews.buttons.render().el);
      }
      this.bindValidation();
      return this;
    };

    return Form;

  })(Backbone.Poised.View);

}).call(this);
