(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  Backbone.Poised.LinkedControl = (function(superClass) {
    extend(LinkedControl, superClass);

    function LinkedControl() {
      this.clone = bind(this.clone, this);
      this.render = bind(this.render, this);
      this.updateLinkIcon = bind(this.updateLinkIcon, this);
      this.toggleLinkState = bind(this.toggleLinkState, this);
      this.hintValidation = bind(this.hintValidation, this);
      return LinkedControl.__super__.constructor.apply(this, arguments);
    }

    LinkedControl.prototype.template = _.template("<div class=\"info\"><label><%= label %></label></div><div class=\"first control\"><div class=\"hint\"></div></div><div class=\"link icon\"></div><div class=\"second control\"><div class=\"hint\"></div></div>");

    LinkedControl.prototype.events = {
      'tap .link.icon': 'toggleLinkState'
    };

    LinkedControl.prototype.initialize = function(options) {
      if (options == null) {
        options = {};
      }
      this.initOptions = options;
      if (options.model1 == null) {
        throw new Error('Missing `model1` option');
      }
      if (options.model2 == null) {
        throw new Error('Missing `model2` option');
      }
      if (options.attribute == null) {
        throw new Error('Missing `attribute` option');
      }
      this.attribute = options.attribute;
      this.model1 = options.model1;
      this.model2 = options.model2;
      this.label = options.label;
      this.stateDefaults = _.chain(options).pick('isLinked').defaults({
        isLinked: true
      }).value();
      if (options.validate != null) {
        this.listenTo(this.model1, 'validated', this.hintValidation);
      }
      this.listenTo(this.model1, "change:" + this.attribute, this.updateLinkedAttribute(this.model2));
      this.listenTo(this.model2, "change:" + this.attribute, this.updateLinkedAttribute(this.model1));
      this.linkState = new Backbone.Model(this.stateDefaults);
      this.linkState.on('change:isLinked', this.updateLinkIcon);
      this.on('liveChangeStart', (function(_this) {
        return function() {
          if (_this.parentView != null) {
            return _this.parentView.trigger('controlLiveChangeStart', _this);
          }
        };
      })(this));
      this.on('liveChange', (function(_this) {
        return function() {
          if (_this.parentView != null) {
            return _this.parentView.trigger('controlLiveChange', _this);
          }
        };
      })(this));
      return this.on('liveChangeEnd', (function(_this) {
        return function() {
          if (_this.parentView != null) {
            return _this.parentView.trigger('controlLiveChangeEnd', _this);
          }
        };
      })(this));
    };

    LinkedControl.prototype.hintValidation = function(isValid, model, errors) {
      var attributeValid;
      attributeValid = this.model.isValid(this.attribute);
      if (errors !== this.lastErrors) {
        this.$el.toggleClass('invalid', !attributeValid);
        this.$hint = this.$('.hint');
        this.$hint.attr({
          'data-error': errors[this.attribute]
        });
      }
      return this.lastErrors = errors;
    };

    LinkedControl.prototype.isLinked = function() {
      return this.linkState.get('isLinked');
    };

    LinkedControl.prototype.toggleLinkState = function() {
      return this.linkState.set({
        isLinked: !this.isLinked()
      });
    };

    LinkedControl.prototype.updateLinkedAttribute = function(linkedModel) {
      return (function(_this) {
        return function(model, value) {
          if (_this.isLinked()) {
            return linkedModel.set(_this.attribute, value);
          }
        };
      })(this);
    };

    LinkedControl.prototype.updateLinkIcon = function() {
      var ref;
      return (ref = this.$link) != null ? ref.toggleClass('unlinked', !this.isLinked()) : void 0;
    };

    LinkedControl.prototype.render = function() {
      this.$el.attr('class', "poised linked-control " + this.attribute);
      this.$el.html(this.template({
        label: this.label || this.loadLocale("formFields." + this.attribute + ".label", {
          defaultValue: this.attribute.toLabel()
        })
      }));
      this.$link = this.$('.link.icon');
      this.$control1 = this.$('.first.control');
      this.$control1.toggleClass(this.attribute);
      this.$control2 = this.$('.second.control');
      this.$control2.toggleClass(this.attribute);
      this.updateLinkIcon();
      return this;
    };

    LinkedControl.prototype.clone = function(options) {
      if (options == null) {
        options = {};
      }
      options = _.chain(options).defaults({
        isLinked: this.isLinked()
      }).defaults(this.initOptions).value();
      return new this.__proto__.constructor(options);
    };

    return LinkedControl;

  })(Backbone.Poised.View);

}).call(this);
