(function() {
  var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  Backbone.Poised.Selectbox = (function(superClass) {
    extend(Selectbox, superClass);

    function Selectbox() {
      this.render = bind(this.render, this);
      this.updateSelectedOption = bind(this.updateSelectedOption, this);
      this.changeAttributeValue = bind(this.changeAttributeValue, this);
      this.initialize = bind(this.initialize, this);
      return Selectbox.__super__.constructor.apply(this, arguments);
    }

    Selectbox.prototype.className = 'poised selectbox';

    Selectbox.prototype.events = {
      'change select': 'changeAttributeValue'
    };

    Selectbox.prototype.initialize = function(options) {
      if (options == null) {
        options = {};
      }
      if (options.model == null) {
        throw new Error('Missing `model` option');
      }
      if (options.attribute == null) {
        throw new Error('Missing `attribute` option');
      }
      this.attribute = options.attribute;
      this.options = _.chain(options).pick('placeholder', 'options', 'multiselect', 'validate').defaults({
        options: [],
        multiselect: false,
        validate: true
      }).value();
      if (this.options.placeholder == null) {
        this.options.placeholder = !this.options.multiselect && (this.model.get(this.attribute) == null);
      }
      if (this.options.placeholder === true) {
        this.options.placeholder = 'Please select ...';
      }
      return this.model.on("change:" + this.attribute, this.updateSelectedOption);
    };

    Selectbox.prototype.changeAttributeValue = function() {
      var option, options;
      if (this.options.multiselect) {
        options = (function() {
          var i, len, ref, results;
          ref = this.$select.find('option:selected');
          results = [];
          for (i = 0, len = ref.length; i < len; i++) {
            option = ref[i];
            results.push($(option).data('value'));
          }
          return results;
        }).call(this);
      } else {
        options = this.$select.find('option:selected').data('value');
      }
      return this.model.set(this.attribute, options);
    };

    Selectbox.prototype.updateSelectedOption = function(model, value) {
      var $option, i, idx, isSelected, len, option, ref, results, selectedValues;
      selectedValues = _.compact(_.flatten([value || this.model.get(this.attribute)]));
      ref = this.$select.find('option');
      results = [];
      for (idx = i = 0, len = ref.length; i < len; idx = ++i) {
        option = ref[idx];
        $option = $(option);
        if (this.options.placeholder && _.isEmpty(selectedValues)) {
          isSelected = idx === 0;
        } else {
          isSelected = _.contains(selectedValues, $option.data('value'));
        }
        results.push($option.attr({
          selected: isSelected
        }));
      }
      return results;
    };

    Selectbox.prototype.render = function() {
      var label, option, options;
      this.$select = $('<select />', {
        "class": 'poised',
        name: this.attribute,
        multiple: this.options.multiselect
      });
      if (this.options.placeholder) {
        this.$select.append($('<option>', {
          text: this.loadLocale("formFields." + this.attribute + ".options.placeholder", "formFields.selectPlaceholder", "generic.selectPlaceholder", {
            defaultValue: this.options.placeholder
          }),
          disabled: true
        }));
      }
      if (_.isFunction(this.options.options)) {
        options = this.options.options.apply(this.model);
      } else {
        options = this.options.options;
      }
      if (_.isArray(options)) {
        options = _.object(options, options);
      }
      for (label in options) {
        option = options[label];
        this.$select.append($('<option>', {
          data: {
            value: option
          },
          text: this.loadLocale("formFields." + this.attribute + ".options." + label, {
            defaultValue: label
          })
        }));
      }
      this.$el.html(this.$select);
      this.updateSelectedOption();
      return this;
    };

    return Selectbox;

  })(Backbone.Poised.View);

}).call(this);
