(function() {
  var base,
    bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  (base = Backbone.Poised).List || (base.List = {});

  Backbone.Poised.List.Item = (function(superClass) {
    extend(Item, superClass);

    function Item() {
      this.render = bind(this.render, this);
      this.updateLabel = bind(this.updateLabel, this);
      this.eventFn = bind(this.eventFn, this);
      this.filter = bind(this.filter, this);
      return Item.__super__.constructor.apply(this, arguments);
    }

    Item.prototype.tagName = 'li';

    Item.prototype.className = 'item';

    Item.prototype.template = _.template('<label class="item"></label><div class="buttons"></div>');

    Item.prototype.initialize = function(options) {
      var changeTrigger;
      if (options == null) {
        options = {};
      }
      Item.__super__.initialize.apply(this, arguments);
      if (options.itemTemplate) {
        this.template = options.itemTemplate;
      }
      this.options = _.pick(options, 'itemLabel');
      if (this.options.itemLabel && _.isFunction(this.options.itemLabel)) {
        changeTrigger = 'change';
        this.options.itemLabel(_.defaults(this.labelParameters(), this.model.toJSON()));
      } else if (this.options.itemLabel && _.isString(this.options.itemLabel)) {
        changeTrigger = "change:" + this.options.itemLabel;
      } else {
        changeTrigger = 'change:label change:title change:name';
      }
      this.model.on(changeTrigger, this.updateLabel);
      return this.parentView.on('filter', this.filter);
    };

    Item.prototype.filter = function(text, sharedObject) {
      var textChar, textChars;
      textChars = text.split('');
      if (sharedObject.fuzzyRegExp == null) {
        sharedObject.fuzzyRegExp = new RegExp(textChars.join('.*'), 'i');
      }
      if (sharedObject.markRegExps == null) {
        sharedObject.markRegExps = (function() {
          var i, len, results;
          results = [];
          for (i = 0, len = textChars.length; i < len; i++) {
            textChar = textChars[i];
            results.push(new RegExp(textChar, 'i'));
          }
          return results;
        })();
      }
      if (text.length === 0 || sharedObject.fuzzyRegExp.test(this.$el.text())) {
        this.$el.css({
          display: ''
        });
      } else {
        this.$el.css({
          display: 'none'
        });
      }
      return this.renderFilterHint(sharedObject.markRegExps);
    };

    Item.prototype.renderFilterHint = function(markRegExps) {
      var i, len, markRegExp, results;
      this.$el.removeMark();
      results = [];
      for (i = 0, len = markRegExps.length; i < len; i++) {
        markRegExp = markRegExps[i];
        results.push(this.$el.markRegExp(markRegExp));
      }
      return results;
    };

    Item.prototype.eventFn = function(name) {
      var eventName;
      eventName = name;
      return function(e) {
        return this.parentView.trigger(eventName, this.model);
      };
    };

    Item.prototype.labelParameters = function() {
      return {};
    };

    Item.prototype.updateLabel = function() {
      var label;
      label = this.options.itemLabel && _.isFunction(this.options.itemLabel) ? this.options.itemLabel(_.defaults(this.labelParameters(), this.model.toJSON())) : this.options.itemLabel && _.isString(this.options.itemLabel) ? this.model.get(this.options.itemLabel) : this.model.get('label') || this.model.get('title') || this.model.get('name') || 'n/A';
      return this.$label.html(label);
    };

    Item.prototype.render = function() {
      this.$el.html(this.template());
      this.$label = this.$('label.item');
      this.updateLabel();
      this.$buttons = this.$('.buttons');
      return this;
    };

    return Item;

  })(Backbone.Poised.View);

}).call(this);
