(function() {
  var markupTextCache;

  markupTextCache = {};

  this.MarkupText = (function() {
    MarkupText.get = function(markup) {
      return markupTextCache[markup] || (markupTextCache[markup] = new MarkupText(markup));
    };

    MarkupText.render = function(context, markup, font) {
      var boundingBox, markupText, tempCanvas, x, y;
      if (context.pixelRatio == null) {
        context.pixelRatio = 1;
      }
      markupText = MarkupText.get(markup);
      tempCanvas = markupText.render(context, font);
      boundingBox = markupText.boundingBox;
      x = (function() {
        switch (context.textAlign) {
          case 'center':
            return Math.floor(boundingBox.w / 2);
          case 'right':
          case 'end':
            return boundingBox.maxX;
          case 'left':
          case 'start':
            return boundingBox.x;
          default:
            return 0;
        }
      })();
      y = (function() {
        switch (context.textBaseline) {
          case 'alphabetic':
            return boundingBox.maxY;
          case 'top':
            return boundingBox.y;
          case 'hanging':
            return boundingBox.y;
          case 'middle':
            return boundingBox.y + Math.floor(boundingBox.h / 2);
          case 'ideographic':
            return boundingBox.maxY;
          case 'bottom':
            return boundingBox.maxY;
          default:
            return 0;
        }
      })();
      context.scale(1 / context.pixelRatio, 1 / context.pixelRatio);
      context.drawImage(tempCanvas, -x, -y);
      return boundingBox;
    };

    function MarkupText(markup) {
      this.markup = markup;
      markupTextCache[markup] = this;
    }

    MarkupText.prototype.getBoundingBox = function(context, alphaThreshold) {
      var alpha, data, h, i, j, maxX, maxY, minX, minY, ref, ref1, w, x, y;
      if (alphaThreshold === void 0) {
        alphaThreshold = 15;
      }
      minX = 2e308;
      minY = 2e308;
      maxX = -2e308;
      maxY = -2e308;
      w = context.canvas.width;
      h = context.canvas.height;
      data = context.getImageData(0, 0, w, h).data;
      for (x = i = 1, ref = w; 1 <= ref ? i < ref : i > ref; x = 1 <= ref ? ++i : --i) {
        for (y = j = 1, ref1 = h; 1 <= ref1 ? j < ref1 : j > ref1; y = 1 <= ref1 ? ++j : --j) {
          alpha = data[(w * y + x) * 4 + 3];
          if (alpha > alphaThreshold) {
            if (x > maxX) {
              maxX = x;
            }
            if (x < minX) {
              minX = x;
            }
            if (y > maxY) {
              maxY = y;
            }
            if (y < minY) {
              minY = y;
            }
          }
        }
      }
      return {
        x: minX,
        y: minY,
        maxX: maxX,
        maxY: maxY,
        w: maxX - minX,
        h: maxY - minY
      };
    };

    MarkupText.prototype.render = function(originalContext, font) {
      if (font == null) {
        font = originalContext != null ? originalContext.font : void 0;
      }
      return this.canvas || (this.canvas = (function(_this) {
        return function() {
          var canvas, context, h, htmlTags, i, len, match, part, ref, ref1, subFontSize, supFontSize, tag, text, x;
          canvas = document.createElement('canvas');
          canvas.width = 300 * originalContext.pixelRatio;
          canvas.height = 300 * originalContext.pixelRatio;
          context = canvas.getContext('2d');
          context.scale(originalContext.pixelRatio, originalContext.pixelRatio);
          x = 0;
          h = parseInt((originalContext != null ? (ref = originalContext.font) != null ? (ref1 = ref.match(/(\d+)px/)) != null ? ref1[1] : void 0 : void 0 : void 0) || 12);
          htmlTags = _this.markup.match(/([^<]+)|<([^>]+)>([^<]*)<\/([^>]+)>/g);
          if (htmlTags != null) {
            for (i = 0, len = htmlTags.length; i < len; i++) {
              part = htmlTags[i];
              context.fillStyle = originalContext != null ? originalContext.fillStyle : void 0;
              context.font = font;
              tag = null;
              text = part;
              if ((match = part.match(/<([^>]+)>([^<]*)<\/([^>]+)>/))) {
                tag = match[1];
                text = match[2];
              }
              if (tag === 'sub') {
                subFontSize = Math.round(parseInt(context.font.match(/\d+/)[0]) * 0.8);
                context.font = context.font.replace(/\d+/, subFontSize);
                context.textBaseline = 'hanging';
              } else if (tag === 'sup') {
                supFontSize = Math.round(parseInt(context.font.match(/\d+/)[0]) * 0.8);
                context.font = context.font.replace(/\d+/, supFontSize);
                context.textBaseline = 'alphabetic';
              } else {
                context.textBaseline = 'middle';
              }
              context.fillText(text, x, h);
              x += context.measureText(text).width + 1;
            }
          }
          _this.boundingBox = _this.getBoundingBox(context);
          return canvas;
        };
      })(this)());
    };

    return MarkupText;

  })();

}).call(this);
